unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, XMLDoc, XMLIntf, ASGSQLite3,  IniFile;

type
  TForm1 = class(TForm)
    Edit1: TEdit;
    Label1: TLabel;
    Label2: TLabel;
    Edit2: TEdit;
    Button1: TButton;
    Label3: TLabel;
    Edit3: TEdit;
    Label4: TLabel;
    Edit4: TEdit;
    procedure FormCreate(Sender: TObject);
    procedure Button1Click(Sender: TObject);
  private
    { Private-Deklarationen }
  public
    { Public-Deklarationen }
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}

function TimeToStr(t: TDateTime): String;
var
  h, m, s, ms: Word;
begin
  DecodeTime(t, h, m, s, ms);
  result := Format('%.2d:%.2d:%.2d', [h,m,s]);
end;

function DateToStr(t: TDateTime): String;
var
  y, m, d: Word;
begin
  DecodeDate(t, y, m, d);
  result := Format('%.4d-%.2d-%.2d', [y, m, d]);
end;

function StrToTime(a: String): TDateTime;
var
  h, m, s: Word;
begin
  h := StrToInt(Copy(a, 1, 2));
  m := StrToInt(Copy(a, 4, 2));
  s := StrToInt(Copy(a, 7, 2));
  result := EncodeTime(h, m, s, 0);
end;

function StrToDate(a: String): TDateTime;
var
  y, m, d: Word;
begin
  y := StrToInt(Copy(a, 1, 4));
  m := StrToInt(Copy(a, 6, 2));
  d := StrToInt(Copy(a, 9, 2));
  result := EncodeDate(y, m, d);
end;

function Default(NodeValue: OleVariant; Default: OleVariant): OleVariant;
begin
  if VarIsNull(NodeValue) then
    result := default
  else
    result := NodeValue;
end;

procedure ConvertDB(source: String; dest: String; realms: TStringList; defaultRealm: Integer; parseExceptions: String);
var
  x: IXMLDocument;
  db: TASQLite3DB;
  sql: TASQLite3Query;
  table: TASQLite3Table;
  i, j: Integer;
  s: String;
  defaultRealmID: Integer;
  gameID: Integer;
  realmID: Integer;
begin
  deletefile(dest);
  db := TASQLite3DB.Create(Application);
  sql := TASQLite3Query.Create(Application);
  table := TASQLite3Table.Create(Application);
  try
    sql.SQLiteDateFormat := true;
    sql.RawSQL := false;
    table.AutoCalcFields := true;
    table.PrimaryAutoInc := true;
    db.Database := dest;
    db.connected := true;
    db.StartTransaction;
    db.SQLite3_ExecSQL('CREATE TABLE players (id INTEGER PRIMARY KEY AUTOINCREMENT,' +
                       'nick VARCHAR NOT NULL,' +
                       'datetime DATETIME,' +
                       'realm INTEGER NOT NULL)');
    db.SQLite3_ExecSQL('CREATE TABLE realms (id INTEGER PRIMARY KEY AUTOINCREMENT,' +
                       'name VARCHAR NOT NULL,' +
                       'server VARCHAR NOT NULL)');
    db.SQLite3_ExecSQL('CREATE TABLE scores (id INTEGER PRIMARY KEY AUTOINCREMENT,'+
                       'playerid INTEGER NOT NULL,' +
                       'gameid INTEGER NOT NULL,' +
                       'slot INTEGER NOT NULL,' +
                       'score INTEGER NOT NULL)');
    db.SQLite3_ExecSQL('CREATE TABLE bans (id INTEGER PRIMARY KEY AUTOINCREMENT,' +
                       'playerid INTEGER NOT NULL,' +
                       'comment VARCHAR NOT NULL,' +
                       'datetime DATETIME,' +
                       'shared BOOLEAN)');
    db.SQLite3_ExecSQL('CREATE TABLE games (id INTEGER PRIMARY KEY AUTOINCREMENT,' +
                       'globalid VARCHAR,' +
                       'ownid INTEGER NOT NULL,' +
                       'hostid INTEGER NOT NULL,' +
                       'realmid INTEGER NOT NULL,' +
                       'mapid INTEGER NOT NULL,' +
                       'gamename VARCHAR,' +
                       'replay VARCHAR,' +
                       'duration VARCHAR,' +
                       'gamestart DATETIME,' +
                       'comment VARCHAR)');
    db.SQLite3_ExecSQL('CREATE TABLE maps (id INTEGER PRIMARY KEY AUTOINCREMENT,' +
                       'mapname VARCHAR NOT NULL,' +
                       'colormapping INTEGER,' +
                       'players INTEGER)');
    db.SQLite3_ExecSQL('CREATE TABLE colormappings (id INTEGER PRIMARY KEY AUTOINCREMENT,' +
                       'name VARCHAR NOT NULL,' +
                       'mapping VARCHAR)');
    sql.Connection := db;
    table.Connection := db;
    // -------------------------- Create Colormaptable ---------------------- //
    table.TableName := 'colormappings';
    table.Open;
    table.Append;
    table.FieldByName('name').AsString := 'default';
    table.FieldByName('mapping').AsString := '1,2,3,4,5,6,7,8,9,10,11,12';
    table.Post;
    table.Close;
    // -------------------------- Create Realmstable ------------------------ //
    table.TableName := 'realms';
    table.Open;
    for i := 0 to realms.Count - 1 do
    begin
      if i = defaultRealm then s := TeilString(realms[i], 1);
      table.Append;
      table.FieldByName('name').AsString := TeilString(realms[i], 1, '|');
      table.FieldByName('server').AsString := TeilString(realms[i], 2, '|');
      table.Post;
    end;
    table.Close;
    sql.SQL.Text := 'SELECT * FROM realms WHERE name = :name';
    sql.Params.ParamByName('name').Value := s;
    sql.ExecSQL;
    sql.First;
    defaultRealmID := sql.FieldValues['id'];
    if fileExists(Source) then
    begin
      x := LoadXMLDocument(source);
      // -------------------------- Create Playertable ------------------------ //
      table.TableName := 'players';
      table.Open;
      with x.DocumentElement.ChildNodes['scorelist'] do
      begin
        for i := 0 to ChildNodes.Count - 1 do
        with ChildNodes[i] do
        begin
          table.Append;
          table.FieldByName('nick').AsString := Attributes['nick'];
          table.FieldByName('datetime').AsDateTime := StrToDate(ChildNodes['lastseen'].ChildNodes['date'].NodeValue) + StrToTime(ChildNodes['lastseen'].ChildNodes['time'].NodeValue);
          table.FieldByName('realm').AsInteger := defaultRealmID;
          table.Post;
        end;
      end;
      with x.DocumentElement.ChildNodes['banlist'] do
      begin
        for i := 0 to ChildNodes.Count - 1 do
        with ChildNodes[i] do
        begin
          sql.SQL.Text := 'SELECT * FROM players WHERE LOWER(nick) = LOWER("'+Attributes['nick']+'")';
          sql.ExecSQL;
          if sql.RecordCount = 0 then
          begin
            table.Append;
            table.FieldByName('nick').AsString := Attributes['nick'];
            table.FieldByName('datetime').AsDateTime := StrToDate(ChildNodes['lastchange'].ChildNodes['date'].NodeValue) + StrToTime(ChildNodes['lastchange'].ChildNodes['time'].NodeValue);
            table.FieldByName('realm').AsInteger := defaultRealmID;
            table.Post;
          end;
        end;
      end;
      with x.DocumentElement.ChildNodes['infolist'] do
      begin
        for i := 0 to ChildNodes.Count - 1 do
        with ChildNodes[i] do
        begin
          sql.SQL.Text := 'SELECT * FROM players WHERE LOWER(nick) = LOWER("'+Attributes['nick']+'")';
          sql.ExecSQL;
          if sql.RecordCount = 0 then
          begin
            table.Append;
            table.FieldByName('nick').AsString := Attributes['nick'];
            table.FieldByName('datetime').AsDateTime := StrToDate(ChildNodes['lastchange'].ChildNodes['date'].NodeValue) + StrToTime(ChildNodes['lastchange'].ChildNodes['time'].NodeValue);
            table.FieldByName('realm').AsInteger := defaultRealmID;
            table.Post;
          end;
        end;
      end;
      with x.DocumentElement.ChildNodes['gamelist'] do
      begin
        for i := 0 to ChildNodes.Count - 1 do
        with ChildNodes[i].ChildNodes['players'] do
        for j := 0 to Childnodes.Count - 1 do
        with ChildNodes[j] do
        if (NodeValue <> 'Computer') and (NodeValue <> 'Open') and (NodeValue <> 'Closed') then
        begin
          sql.SQL.Text := 'SELECT players.id, name FROM players,realms WHERE LOWER(nick) = LOWER("'+NodeValue+'") and players.realm = realms.id and LOWER(realms.name) = LOWER("'+x.DocumentElement.ChildNodes['gamelist'].ChildNodes[i].ChildNodes['realm'].NodeValue+'")';
          sql.ExecSQL;
          if (sql.RecordCount = 0) then
          begin
            table.Append;
            table.FieldByName('nick').AsString := NodeValue;
            with x.DocumentElement.ChildNodes['gamelist'].ChildNodes[i].ChildNodes['gamestart'] do
              table.FieldByName('datetime').AsDateTime := StrToDate(ChildNodes['date'].NodeValue) + StrToTime(ChildNodes['time'].NodeValue);
            sql.SQL.Text := 'SELECT * FROM realms WHERE LOWER(name) = LOWER("'+ x.DocumentElement.ChildNodes['gamelist'].ChildNodes[i].ChildNodes['realm'].NodeValue+'")';
            sql.ExecSQL;
            if sql.RecordCount = 0 then
              table.FieldByName('realm').AsInteger := defaultRealmID
            else
              table.FieldByName('realm').AsInteger := sql.FieldByName('id').AsInteger;
            table.Post;
          end;
        end;
      end;
      table.Close;
      // -------------------------- Create Bantable --------------------------- //
      table.TableName := 'bans';
      table.Open;
      with x.DocumentElement.ChildNodes['banlist'] do
      begin
        for i := 0 to ChildNodes.Count - 1 do
        with ChildNodes[i] do
        if Attributes['deleted'] = 0 then
        begin
          sql.SQL.Text := 'SELECT * FROM players WHERE LOWER(nick) = LOWER("'+Attributes['nick']+'")';
          sql.ExecSQL;
          begin
            table.Append;
            table.FieldByName('playerid').AsInteger := sql.FieldValues['id'];
            table.FieldByName('comment').AsString := Default(ChildNodes['comment'].NodeValue, '');
            table.FieldByName('datetime').AsDateTime := StrToDate(ChildNodes['lastchange'].ChildNodes['date'].NodeValue) + StrToTime(ChildNodes['lastchange'].ChildNodes['time'].NodeValue);
            table.FieldByName('shared').AsBoolean := true;
            table.Post;
          end;
        end;
      end;
      with x.DocumentElement.ChildNodes['infolist'] do
      begin
        for i := 0 to ChildNodes.Count - 1 do
        with ChildNodes[i] do
        if Attributes['deleted'] = 0 then
        begin
          sql.SQL.Text := 'SELECT * FROM players WHERE LOWER(nick) = LOWER("'+Attributes['nick']+'")';
          sql.ExecSQL;
          begin
            table.Append;
            table.FieldByName('playerid').AsInteger := sql.FieldValues['id'];
            table.FieldByName('comment').AsString := Default(ChildNodes['comment'].NodeValue, '');
            table.FieldByName('datetime').AsDateTime := StrToDate(ChildNodes['lastchange'].ChildNodes['date'].NodeValue) + StrToTime(ChildNodes['lastchange'].ChildNodes['time'].NodeValue);
            table.FieldByName('shared').AsBoolean := false;
            table.Post;
          end;
        end;
      end;
      table.Close;
      // -------------------------- Create Maptable --------------------------- //
      table.TableName := 'maps';
      table.Open;
      with x.DocumentElement.ChildNodes['gamelist'] do
      begin
        for i := 0 to ChildNodes.Count - 1 do
        with ChildNodes[i] do
        if Attributes['id'] <> '' then
        begin
          sql.SQL.Text := 'SELECT * FROM maps WHERE LOWER(mapname) = LOWER("' + ChildNodes['mapname'].NodeValue + '")';
          sql.ExecSQL;
          if sql.RecordCount = 0 then
          begin
            table.Append;
            table.FieldByName('mapname').AsString := ChildNodes['mapname'].NodeValue;
            table.FieldByName('colormapping').AsInteger := 1;
            table.FieldByName('players').AsInteger := 0;
            table.Post;
          end;
        end;
      end;
      table.Close;
      // -------------------------- Create Gametable -------------------------- //
      table.TableName := 'games';
      table.Open;
      with x.DocumentElement.ChildNodes['gamelist'] do
      begin
        for i := 0 to ChildNodes.Count - 1 do
        with ChildNodes[i] do
        if Attributes['id'] <> '' then
        begin
          sql.SQL.Text := 'SELECT * FROM realms WHERE name = "' + ChildNodes['realm'].NodeValue + '"';
          sql.ExecSQL;
          if sql.RecordCount > 0 then
          begin
            table.Append;
            table.FieldByName('globalid').asString := Attributes['id'];
            table.FieldByName('realmid').AsInteger := sql.fieldByName('id').AsInteger;
            sql.SQL.Text := 'SELECT id FROM players WHERE LOWER(nick) = LOWER("' + ChildNodes['ownnickname'].NodeValue + '") and realm = '+IntToStr(table.fieldByName('realmid').AsInteger);
            sql.ExecSQL;
            table.FieldByName('ownid').AsInteger := sql.fieldbyname('id').AsInteger;
            if ChildNodes['host'].NodeValue = Null then
              table.FieldByName('hostid').AsInteger := 0
            else begin
              sql.SQL.Text := 'SELECT id FROM players WHERE LOWER(nick) = LOWER("' + ChildNodes['host'].NodeValue + '") and realm = '+IntToStr(table.fieldByName('realmid').AsInteger);
              sql.ExecSQL;
              table.FieldByName('hostid').AsInteger := sql.fieldbyname('id').AsInteger;
            end;
            table.FieldByName('gamename').AsString := ChildNodes['gamename'].NodeValue;
            table.FieldByName('duration').AsString := ChildNodes['duration'].NodeValue;
            table.FieldByName('replay').AsString := Default(ChildNodes['replay'].NodeValue, '');
            sql.SQL.Text := 'SELECT id FROM maps WHERE LOWER(mapname) = LOWER("' + ChildNodes['mapname'].NodeValue + '")';
            sql.ExecSQL;
            table.FieldByName('mapid').AsInteger := sql.fieldByName('id').AsInteger;
            table.FieldByName('comment').AsString := Default(ChildNodes['comment'].NodeValue, '');
            table.FieldByName('gamestart').AsDateTime := StrToDate(ChildNodes['gamestart'].ChildNodes['date'].NodeValue) + StrToTime(ChildNodes['gamestart'].ChildNodes['time'].NodeValue);
            table.Post;
          end;
        end;
      end;
      table.Close;
      // -------------------------- Create Scoretable ------------------------- //
      table.TableName := 'scores';
      table.Open;
      with x.DocumentElement.ChildNodes['gamelist'] do
      begin
        for i := 0 to ChildNodes.Count - 1 do
        with ChildNodes[i] do
        if Attributes['id'] <> '' then
        begin
          sql.SQL.Text := 'SELECT * FROM games WHERE globalid = "' + Attributes['id'] + '"';
          sql.ExecSQL;
          gameID := sql.fieldbyname('id').AsInteger;
          realmID := sql.fieldbyname('realmid').AsInteger;
          for j := 0 to ChildNodes['players'].ChildNodes.Count - 1 do
          begin
            s := ChildNodes['players'].ChildNodes[j].NodeValue;
            if (s <> 'Computer') and (s <> 'Open') and (s <> 'Closed') then
            begin
              sql.SQL.Text := 'SELECT id from players WHERE realm = ' + IntToStr(realmID) + ' AND LOWER(nick) = LOWER("'+s+'")';
              sql.ExecSQL;
              if sql.RecordCount > 0 then
              begin
                table.Append;
                table.FieldByName('playerid').AsInteger := sql.fieldbyname('id').AsInteger;
                table.FieldByName('gameid').AsInteger := gameID;
                table.FieldByName('score').AsInteger := ChildNodes['givenscores'].ChildNodes[j].NodeValue;
                table.FieldByName('slot').AsInteger := j + 1;              
                table.Post;
              end;
            end;
          end;
        end;
      end;
      table.Close;
      x := nil;
    end;
    // ------------------------- Load number of human players into DB ------- //
    if fileexists(parseExceptions) then
    begin
      x := LoadXMLDocument(parseExceptions);
      for i := 0 to x.DocumentElement.ChildNodes.Count - 1 do
      with x.DocumentElement.ChildNodes[i] do
      begin
        sql.SQL.Text := 'SELECT * FROM maps WHERE LOWER(mapname) = LOWER("' + Attributes['filename'] + '")';
        sql.ExecSQL;
        if sql.RecordCount = 0 then
        begin
          table.TableName := 'maps';
          table.Open;
          table.Append;
          table.FieldByName('mapname').AsString := Attributes['filename'];
          table.FieldByName('players').AsInteger := Attributes['humanplayers'];
          table.FieldByName('colormapping').AsInteger := 1;
          table.Post;
        end else
        begin
          sql.SQL.Text := 'UPDATE maps SET players = ' + Attributes['humanplayers'] + ' WHERE id = ' + IntToStr(sql.fieldbyname('id').AsInteger);
          sql.ExecSQL;
        end;
      end;
      x := nil;
    end;
    db.Commit;
  finally
    freeandnil(sql);
    freeandnil(table);
    freeandnil(db);
  end;
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
  edit2.Text := ExtractFilePath(Application.ExeName) + 'local.sdb';
end;

procedure TForm1.Button1Click(Sender: TObject);
var
  ms: DWORD;
  realms: TStringList;
begin
  ms := GetTickCount;
  realms := TStringlist.Create;
  realms.LoadFromFile(edit3.Text);
  ConvertDB(edit1.text, edit2.text, realms, 3, edit4.text);
  realms.Free;
  ShowMessage('Dauer: ' + IntToStr(GetTickCount - ms));
end;

end.
