unit HotKeyReceiver;

interface

uses
  Classes, Forms, Messages, Windows, SysUtils;

const
  MMFName = 'WC3BanlistHotkeyHook_Data';
  keybmsg = 'WC3BanlistHotkeyHook_Msg';
  keysem_ready = 'WC3BanlistHotKeyHook_KeyReady';
  keysem_access = 'WC3BanlistHotKeyHook_KeyAccess';
  SEMAPHORE_ALL_ACCESS = STANDARD_RIGHTS_REQUIRED or SYNCHRONIZE or 3;

type
  TDLLData = packed record
    ProcessID, ThreadID: DWORD;
    nCode: Integer;
    wParam: WPARAM;
    lParam: LPARAM;
  end;
  PDLLData = ^TDLLData;

  THotKeyData = packed record
    ProcessID,
      ThreadID,
      Key: DWORD;
  end;
  PHotKeyData = ^THotKeyData;

  THotKeyReceiver = class(TThread)
  private
    lpView: PDLLData;
    hReady, hAccess: THandle;
  protected
  public
    ThreadList: TThreadList;
    procedure Clear;
    constructor Create;
    destructor Destroy; override;
    procedure Execute; override;
  end;

implementation

uses
  Dialogs;

function InstallHook(Hwnd: Cardinal): Boolean; stdcall; external
  'HotKeyHook.dll';

function UninstallHook: Boolean; stdcall; external 'HotKeyHook.dll';

function ReturnView: Pointer; stdcall; external 'HotKeyHook.dll';

{ THotKeyReceiver }

constructor THotKeyReceiver.Create;
begin
  inherited Create(true);
  ThreadList := TThreadList.Create;
  InstallHook(Application.Handle);
  lpView := ReturnView;
  hReady := OpenSemaphore(SEMAPHORE_ALL_ACCESS, false, keysem_ready);
  if hReady <> 0 then
  begin
    hAccess := OpenSemaphore(SEMAPHORE_ALL_ACCESS, false, keysem_access);
    if hAccess <> 0 then
      Resume
    else
      CloseHandle(hReady);
  end else
    Exception.Create('Could not create hotkey hook thread.');
end;

destructor THotKeyReceiver.Destroy;
begin
  if (hReady <> 0) and (hAccess <> 0) then
  begin
    Terminate;
    WaitFor;
    CloseHandle(hReady);
    CloseHandle(hAccess);
  end;
  UninstallHook;
  FreeAndNil(ThreadList);
  inherited;
end;

procedure THotKeyReceiver.Clear;
var
  i: Integer;
  list: TList;
begin
  list := ThreadList.LockList;
  for i := 0 to list.Count - 1 do
    Dispose(list[i]);
  list.Clear;
  ThreadList.UnlockList;
end;

procedure THotKeyReceiver.Execute;
var
  HotKey: PHotKeyData;
begin
  while not terminated do
  begin
    if WaitForSingleObject(hReady, 100) <> WAIT_TIMEOUT then
    begin
      WaitForSingleObject(hAccess, INFINITE);
      New(HotKey);
      HotKey^.ProcessID := lpView^.ProcessID;
      HotKey^.ThreadID := lpView^.ThreadID;
      HotKey^.Key := lpView^.wParam;
      ThreadList.Add(HotKey);
      ReleaseSemaphore(hAccess, 1, nil);
    end;
  end;
end;

end.
