program ldbtool;

{$APPTYPE CONSOLE}

uses
  SysUtils,
  Forms,
  Classes,
  ComCtrls;

const tz = ' ';

function TeilString(a: string; n: integer; mitRest: boolean): string; overload;
var
  p, i: integer;
  b: string;
begin
  i := pos(TZ, a);
  if i = 0 then
  begin
    result := a;
    exit;
  end;
  i := 1;
  b := a;
  while i < n do
  begin
    i := i + 1;
    p := pos(TZ, b);
    b := copy(b, p + 1, 2000000);
  end;
  if mitrest or (pos(TZ, b) = 0) then
    result := b
  else
    result := copy(b, 1, pos(TZ, b) - 1);
end;

procedure usage(error: string);
begin
    if error <> '' then
    begin
      writeln(error);
      writeln;
    end else
    begin
      writeln('LOCAL DATABASE TOOL for WC3Banlist');
      writeln('----------------------------------');
      writeln;
    end;
    writeln('Usage:');    //                      0     2
    writeln('    ldbtool {export|import|remove} {bans|infos|both} {database} [text file] [pattern]');
    writeln;
    writeln('Commands:');
    writeln('    export: exports the local database to a text file');
    writeln('    import: imports the text file to the local database');
    writeln('             as one single type of entries');
    writeln('    remove: removes all entries of the given names in the text file');
    writeln;
    writeln('Restrictions:');
    writeln('    "both" is only allowed for export and remove');
    writeln('    on remove only bans of the given type will be removed');
    writeln('    on import existing entries will be overwritten');
    writeln('    on export the file name of the text file can be omitted to print to screen');
    writeln;
    readln;
    halt;
end;

procedure SaveListViewToFile(AListView: TListView; sFileName: string);
var
  idxItem, idxSub, IdxImage: Integer;
  F: TFileStream;
  sText: string;
  W, ItemCount, SubCount: Word;
  MySignature: array[0..2] of Char;
begin
  //Initialization
  with AListView do
  begin
    ItemCount := 0;
    SubCount := 0;
    //****
    MySignature := 'LVF';
    //  ListViewFile
    F := TFileStream.Create(sFileName, fmCreate or fmOpenWrite);
    F.Write(MySignature, SizeOf(MySignature));

    if Items.Count = 0 then
      // List is empty
      ItemCount := 0
    else
      ItemCount := Items.Count;
    F.Write(ItemCount, SizeOf(ItemCount));

    if Items.Count > 0 then
    begin
      for idxItem := 1 to ItemCount do
      begin
        with Items[idxItem - 1] do
        begin
          //Save subitems count
          SubCount := Subitems.Count;
          F.Write(SubCount, SizeOf(SubCount));
          //Save Data
          F.Write(Data, sizeof(Pointer));
          //Save ImageIndex
          IdxImage := ImageIndex;
          F.Write(IdxImage, SizeOf(IdxImage));
          //Save Caption
          sText := Caption;
          w := Length(sText);
          F.Write(w, SizeOf(w));
          if w>0 then
            F.Write(sText[1], w);
          if SubCount > 0 then
          begin
            for idxSub := 0 to SubItems.Count - 1 do
            begin
              //Save Item's subitems
              sText := SubItems[idxSub];
              w := Length(sText);
              F.Write(w, SizeOf(w));
              if w > 0 then
                F.Write(sText[1], w);
            end;
          end;
        end;
      end;
    end;
    F.Free;
  end;
end;

procedure LoadListViewFromFile(AListView: TListView; sFileName: string);
var
  F: TFileStream;
  IdxItem, IdxSubItem, IdxImage: Integer;
  W, ItemCount, SubCount: Word;
  pText: PChar;
  PTemp: PChar;
  Dat: Pointer;
  MySignature: array[0..2] of Char;
  sExeName: string;
begin
  with AListView do
  begin
    ItemCount := 0;
    SubCount := 0;

    sExeName := ExtractFileName(sFileName);

    if not FileExists(sFileName) then
      usage('Database file not found!');

    F := TFileStream.Create(sFileName, fmOpenRead);
    F.Read(MySignature, SizeOf(MySignature));

    if MySignature <> 'LVF' then
      usage('Illegal database file!');

    F.Read(ItemCount, SizeOf(ItemCount));
    Items.BeginUpdate;
    Items.Clear;

    for idxItem := 1 to ItemCount do
    begin
      with Items.Add do
      begin
        //Read Subcount
        F.Read(SubCount, SizeOf(SubCount));
        //Read Data
        F.Read(Dat, SizeOf(pointer));
        Data := Dat;
        //Read imageindex
        F.Read(IdxImage, SizeOf(IdxImage));
        ImageIndex := IdxImage;
        //Read the Caption
        F.Read(w, SizeOf(w));
        if w > 0 then begin
          pText := StrAlloc(w + 1);
          pTemp := StrAlloc(w + 1);
          F.Read(pTemp^, W);
          StrLCopy(pText, pTemp, W);
          Caption := StrPas(pText);
          StrDispose(pTemp);
          StrDispose(pText);
        end else
          Caption := '';
        if SubCount > 0 then
        begin
          for idxSubItem := 1 to SubCount do
          begin
            F.Read(w, SizeOf(w));
            if w > 0 then begin
              pText := StrAlloc(w + 1);
              pTemp := StrAlloc(w + 1);
              F.Read(pTemp^, W);
              StrLCopy(pText, pTemp, W);
              Items[idxItem - 1].SubItems.Add(StrPas(pText));
              StrDispose(pTemp);
              StrDispose(pText);
            end else
              Items[idxItem - 1].SubItems.Add('');
          end;
        end;
      end;
    end;
    Items.EndUpdate;
    F.Free;
  end;
end;

var
  command: string;
  entrytype: string;
  database: string;
  textfile: string;
  list: tstringlist;
  lv: TListView;
  i, j: integer;
  form: TForm;


begin
  if (paramcount < 3) or (paramcount > 4) then usage('');
  command := paramstr(1);
  entrytype := paramstr(2);
  database := paramstr(3);
  if paramcount = 4 then
    textfile := paramstr(4)
  else
    textfile := '';

  { Check parameters }
  if (command <> 'export') and (command <> 'import') and (command <> 'remove') then
    usage('Unknown command given! Must be export, import or remove.');
  if (entrytype <> 'bans') and (entrytype <> 'infos') and (entrytype <> 'both') then
    usage('Unknown entry type given! Must be bans, infos or both.');
  if (textfile = '') and (command <> 'export') then
    usage('You must give a text file for '+command+'!');
  if (entrytype = 'both') and (command = 'import') then
    usage('Entry type "both" is not allowed for import.');

  list := TStringlist.Create;
  if (command = 'import') or (command = 'remove') then
  try
    list.LoadFromFile(textfile);
  except
    on e: Exception do
      usage('Cannot open text file: ' + e.Message);
  end else if textfile <> '' then
  try
    list.SaveToFile(textfile);
  except
    on e: Exception do
      usage('Cannot write output file: ' + e.Message);
  end;

  form := TForm.Create(nil);
  lv := TListView.Create(form);
  lv.Parent := form;
  try
    LoadListViewFromFile(lv, database);
  except
    if (command <> 'export') then
      usage('No database given for import or remove!')
    else
      lv.Clear;
  end;

  if command = 'export' then begin
    for i := 0 to lv.Items.Count - 1 do
    with lv.Items[i] do begin
      if (imageIndex = 0) and ((entrytype = 'bans') or (entrytype = 'both'))
      then
        list.Add(Caption + ' ' + SubItems[0]);
      if (imageIndex = 2) and ((entrytype = 'infos') or (entrytype = 'both'))
      then
        list.Add(Caption + ' ' + SubItems[0]);
    end;
    if (textfile <> '') then
      list.SaveToFile(textfile)
    else
      for i := 0 to list.Count - 1 do writeln(list[i]);
  end else if command = 'remove' then begin
    for i := 0 to list.Count - 1 do begin
      for j := 0 to lv.Items.Count - 1 do
      with lv.Items[j] do
        if lowercase(caption) = teilstring(list[i], 1, false) then
          if (entrytype = 'both') or ((entrytype = 'infos') and (imageindex = 2)) or
            ((entrytype = 'bans') and (imageIndex = '0')) then
          begin
            Delete;
            break;
          end;
    try
      SaveListViewToFile(lv, database);
    except
      Usage('Cannot write database file.');
    end;
  end;

  readln;

end.
