unit FiveNum;

interface

uses
  Classes;

type
  TFiveNumberSummary = record
    Minimum: Integer;
    LowerQuartile: Integer;
    Median: Integer;
    UpperQuartile: Integer;
    Maximum: Integer;
  end;

function SortList(aList: TList): TList;

function MedianRange(aSortedList: TList; aStart: Integer; aEnd: Integer):
  Integer;

function Minimum(aSortedList: TList): Integer;
function LowerQuartile(aSortedList: TList): Integer;
function Median(aSortedList: TList): Integer;
function UpperQuartile(aSortedList: TList): Integer;
function Maximum(aSortedList: TList): Integer;

function FiveNumberSummary(aSortedList: TList): TFiveNumberSummary;

implementation

uses
  SysUtils, Math;

function IntCompare(Item1, Item2: Pointer): Integer;
begin
  Result := Integer(Item1) - Integer(Item2);
end;

function SortList(aList: TList): TList;
begin
  aList.Sort(IntCompare);
  Result := aList;
end;

function MedianRange(aSortedList: TList; aStart: Integer; aEnd: Integer):
  Integer;
begin
  with aSortedList do
  begin
    if Count = 0 then
      raise EMathError.Create('Cannot calculate the median of an empty list!');

    if aStart < 0 then
      aStart := 0;

    if aEnd < 0 then
      aEnd := 0;

    if aEnd > Count - 1 then
      aEnd := Count - 1;

    if aStart > aEnd then
      aStart := aEnd;

    if aStart = aEnd then
      Result := Integer(Items[aStart])
    else
    begin
      if (aEnd - aStart) and 1 = 1 then
        Result := (Integer(Items[(aEnd - aStart + 1) shr 1 + aStart - 1]) +
          Integer(Items[(aEnd - aStart + 1) shr 1 + aStart])) shr 1
      else
        Result := Integer(Items[(aEnd - aStart + 1) shr 1 + aStart]);
    end;
  end;
end;

function Minimum(aSortedList: TList): Integer;
begin
  Result := Integer(aSortedList.First);
end;

function LowerQuartile(aSortedList: TList): Integer;
begin
  Result := MedianRange(aSortedList, 0, aSortedList.Count shr 1 - 1);
end;

function Median(aSortedList: TList): Integer;
begin
  with aSortedList do
  begin
    if Count = 0 then
      raise EMathError.Create('Cannot calculate the median of an empty list!');

    if Count = 1 then
      Result := Integer(First)
    else
    begin
      if Count and 1 = 0 then
        Result := (Integer(Items[Count shr 1 - 1]) +
          Integer(Items[Count shr 1])) shr 1
      else
        Result := Integer(Items[Count shr 1]);
    end;
  end;
end;

function UpperQuartile(aSortedList: TList): Integer;
begin
  with aSortedList do
    Result := MedianRange(aSortedList, Count shr 1 + (Count and 1), Count - 1);
end;

function Maximum(aSortedList: TList): Integer;
begin
  Result := Integer(aSortedList.Last);
end;

function FiveNumberSummary(aSortedList: TList): TFiveNumberSummary;
begin
  Result.Minimum := Minimum(aSortedList);
  Result.LowerQuartile := LowerQuartile(aSortedList);
  Result.Median := Median(aSortedList);
  Result.UpperQuartile := UpperQuartile(aSortedList);
  Result.Maximum := Maximum(aSortedList);
end;

end.
