unit BanIO;

interface

uses Classes, WC3Types, SysUtils;

procedure bans_SaveBans(sFileName: string; l: TList);
procedure bans_LoadBans(sFileName: string; l: TList);
procedure bans_ClearList(l: TList);
function bans_AddItem(l: TList): PBan;

implementation

procedure bans_ClearList(l: TList);
var
  p: PBan;
begin
  while l.count > 0 do
  begin
    p := l[0];
    p^.nickname := '';
    p^.comment := '';
    p^.source := '';
    dispose(p);
    l.Delete(0);
  end;
end;

function bans_AddItem(l: TList): PBan;
var
  p: PBan;
begin
  new(p);
  p^.nickname := '';
  p^.comment := '';
  p^.source := '';
  p^.imageindex := 0;
  p^.data := nil;
  l.Add(p);
  result := p;
end;

procedure bans_SaveBans(sFileName: string; l: TList);
var
  idxItem, IdxImage: Integer;
  F: TFileStream;
  pText: PChar;
  sText: string;
  W, ItemCount, SubCount: Word;
  MySignature: array[0..2] of Char;
  p: PBan;
begin
  //****
  MySignature := 'LVF';
  //  ListViewFile
  F := TFileStream.Create(sFileName, fmCreate or fmOpenWrite);
  F.Write(MySignature, SizeOf(MySignature));

  ItemCount := l.Count;
  F.Write(ItemCount, SizeOf(ItemCount));

  for idxItem := 1 to ItemCount do
  begin
    p := l[idxItem - 1];
    //Save subitems count
    SubCount := 1;
    F.Write(SubCount, SizeOf(SubCount));
    //Save Data
    F.Write(p^.Data, sizeof(Pointer));
    //Save ImageIndex
    IdxImage := p^.imageindex;
    F.Write(IdxImage, SizeOf(IdxImage));

    //Save Caption
    sText := p^.nickname;
    w := Length(sText);
    pText := StrAlloc(Length(sText) + 1);
    StrPLCopy(pText, sText, Length(sText));
    F.Write(w, SizeOf(w));
    F.Write(pText^, w);
    StrDispose(pText);

    //Save Item's subitems
    sText := p^.comment;
    w := Length(sText);
    pText := StrAlloc(Length(sText) + 1);
    StrPLCopy(pText, sText, Length(sText));
    F.Write(w, SizeOf(w));
    F.Write(pText^, w);
    StrDispose(pText);
  end;
  F.Free;
end;

procedure bans_LoadBans(sFileName: string; l: TList);
var
  F: TFileStream;
  IdxItem, IdxImage: Integer;
  W, ItemCount, SubCount: Word;
  pText: PChar;
  PTemp: PChar;
  Dat: Pointer;
  MySignature: array[0..2] of Char;
  p: PBan;
begin
  F := TFileStream.Create(sFileName, fmOpenRead);
  F.Read(MySignature, SizeOf(MySignature));

  if MySignature <> 'LVF' then
    raise Exception.Create('bans_LoadBans: wrong file format');

  F.Read(ItemCount, SizeOf(ItemCount));

  bans_ClearList(l);
  for idxItem := 1 to ItemCount do
  begin
    p := bans_AddItem(l);
    //Read Subcount
    F.Read(SubCount, SizeOf(SubCount));
    //Read Data
    F.Read(Dat, SizeOf(pointer));
    p^.Data := Dat;
    //Read imageindex
    F.Read(IdxImage, SizeOf(IdxImage));
    p^.ImageIndex := IdxImage;
    //Read the nickname
    F.Read(w, SizeOf(w));
    pText := StrAlloc(w + 1);
    pTemp := StrAlloc(w + 1);
    F.Read(pTemp^, W);
    StrLCopy(pText, pTemp, W);
    p^.nickname := StrPas(pText);
    StrDispose(pTemp);
    StrDispose(pText);
    //Read the comment
    F.Read(w, SizeOf(w));
    pText := StrAlloc(w + 1);
    pTemp := StrAlloc(w + 1);
    F.Read(pTemp^, W);
    StrLCopy(pText, pTemp, W);
    p^.comment := StrPas(pText);
    StrDispose(pTemp);
    StrDispose(pText);
  end;
  F.Free;
end;

end.
