unit Diagnosis;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, Clipbrd, OSVersion, UPacket, ExtCtrls, WC3Types, IniFile;

type
  TfrmDiagnosis = class(TForm)
    Memo: TMemo;
    Panel1: TPanel;
    Button1: TButton;
    btnRefresh: TButton;
    btnClose: TButton;
    procedure btnCloseClick(Sender: TObject);
    procedure Button1Click(Sender: TObject);
    procedure RefreshDiagnosis;
    procedure FormShow(Sender: TObject);
    procedure btnRefreshClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    { Private-Deklarationen }
  public
    { Public-Deklarationen }
  end;

var
  frmDiagnosis: TfrmDiagnosis;

implementation

uses Unit1;

{$R *.dfm}

procedure TfrmDiagnosis.btnCloseClick(Sender: TObject);
begin
  close;
end;

procedure TfrmDiagnosis.Button1Click(Sender: TObject);
begin
  Clipboard.AsText := Memo.Text;
end;

function GetVersion(FileName: TFileName): string;
var
  VerInfoSize: DWORD;
  VerInfo: Pointer;
  VerValueSize: DWORD;
  VerValue: PVSFixedFileInfo;
  Dummy: DWORD;
begin
  VerInfoSize := GetFileVersionInfoSize(PChar(FileName), Dummy);
  GetMem(VerInfo, VerInfoSize);
  GetFileVersionInfo(PChar(FileName), 0, VerInfoSize, VerInfo);
  VerQueryValue(VerInfo, '\', Pointer(VerValue), VerValueSize);
  with VerValue^ do
  begin
    Result := IntToStr(dwFileVersionMS shr 16);
    Result := Result + '.' + IntToStr(dwFileVersionMS and $FFFF);
    Result := Result + '.' + IntToStr(dwFileVersionLS shr 16);
    Result := Result + '.' + IntToStr(dwFileVersionLS and $FFFF);
  end;
  FreeMem(VerInfo, VerInfoSize);
end;

procedure TfrmDiagnosis.RefreshDiagnosis;
var
  b: array[0..MAX_PATH - 1] of char;
  i: Integer;
begin
  with Memo.Lines, WC3Banlist_Main do
  begin
    Clear;
    // OS Version
    Add('OS: ' + GetOSVersionText);
    // WinPCap
    getSystemDirectory(b, sizeof(b));
    if fileexists(b + '\packet.dll') then
      Add('WinPCap Version: ' + GetVersion(b + '\packet.dll'))
    else
      Add('WinPCap not installed.');
    if pcapok then
      Add('WinPCap is running properly.')
    else
      Add('WinPCap could not be initialized.');
    if FindWindow('Warcraft III', 'Warcraft III') <> 0 then
      Add('Warcraft 3 found.')
    else
      Add('Warcraft 3 not found. Make sure you run WC3Banlist, then WC3 and log into battle.net before running the diagnostics.');
    Add('WC3Banlist Version: ' + CURRENT_VERSION);  
    Add('');
    // IP Addresses
    Add('Local IP addresses:');
    for i := 0 to ipcount - 1 do
      Add(#32#32 + IPToStr(@myip[i]));
    Add('');
    // Adapter
    Add('Available adapters:');
    for i := 0 to cbNIC.Items.Count - 1 do
      Add(#32#32 + cbNIC.Items[i]);
    if cbNIC.Items.Count = 0 then
      Add(#32#32 + 'none. Looks like WinPCap is not working with your NIC or OS.');
    Add('');
    if pcapsniffing then
      Add('Listening for traffic on '#13#10#32#32 + edtNIC.Text +
        ' for ' + TimeToDuration(GetTickCount - pcapsniffingsince))
    else
      Add('Sniffing failed.');
    Add('');
    // Filter
    if pcapfiltering then
      Add('Filtering traffic for TCP or PPPoE.')
    else
      Add('Filtering not possible.');
    Add('');
    // Traffic
    Add('Traffic occurrence:');
    if trafficPPPoE then
      Add(#32#32 + 'PPPoE packets received.')
    else
      Add(#32#32 + 'No PPPoE packets received yet.');
    if trafficTCP then
      Add(#32#32 + 'TCP packets received.')
    else
      Add(#32#32 + 'No TCP packets received yet.');
    if trafficWC3In then
      Add(#32#32 + 'Incoming WC3 packets received.')
    else
      Add(#32#32 + 'No incoming WC3 packets received yet.');
    if trafficWC3Out then
      Add(#32#32 + 'Outgoing WC3 packets received.')
    else
      Add(#32#32 + 'No outgoing WC3 packets received yet.');
    Add('');
    // Diagnosis
    if trafficWC3In and trafficWC3Out then
      Add('Packet capturing should be working fine.')
    else if trafficWC3In and not trafficWC3Out then
    begin
      Add('WinPCap doesn''t seem to support capturing outgoing packets on your NIC.');
      Add('Some things may be working but others not. :(');
    end
    else if trafficTCP then
    begin
      Add('Some TCP packets have been received but no WC3 traffic.');
      Add('Make sure you have started WC3 and logged on the battle.net before running diagnostics.');
    end
    else if trafficPPPoe then
    begin
      Add('Some PPPoE packets have been received but no TCP traffic.');
      Add('This means that WC3Banlist could not handle the kind of PPPoE used by your NIC.');
      Add('Contact knarf@wc3banlist.de .');
    end
    else if pcapsniffing then
    begin
      Add('WinPCap is listening on the selected adapter but no packets have been received.');
      Add('Make sure you selected the one which connects you to the internet.')
    end
    else if pcapok then
    begin
      if (cbNIC.ItemIndex = -1) and (cbNIC.Items.Count > 0)
      then
        Add('You haven''t selected a NIC yet.')
      else
        Add('WinPCap is installed properly but the selected NIC is not supported.')
    end
    else
    begin
      Add('WinPCap could not be initialized.');
      Add('Make sure you have installed the latest version.');
    end;
    // Antidownload
    if trafficPPPoE then
    begin
      if pos('Service Pack 2', GetOSVersionText) > 0 then
        Add('Anti download and autokick don''t work with PPPoE adapters in connection with SP2.')
      else if WC3Banlist_Main.chkWinPCap.Checked then
      begin
        Add('You have selected WinPCap for joining prevention packet generation.');
        Add('This will cause problems because WinPCap''s packet generation doesn''t work on PPPoE connections.');
        Add('To enable anti download and autokick choose WinSock at Preferences -> NIC Settings.');
      end;
    end
    else if WC3Banlist_Main.chkWinSock.Checked and (pos('Service Pack 2',
      GetOSVersionText) > 0) then
    begin
      Add('You have selected WinSock for joining prevention packet generation.');
      Add('This will cause problems because since SP2 WinSock doesn''t allow the generation of packets.');
      Add('To enable anti download and autokick choose WinPCap at Preferences -> NIC Settings.');
    end;
  end;
end;

procedure TfrmDiagnosis.FormShow(Sender: TObject);
begin
  RefreshDiagnosis;
end;

procedure TfrmDiagnosis.btnRefreshClick(Sender: TObject);
begin
  RefreshDiagnosis;
end;

procedure TfrmDiagnosis.FormCreate(Sender: TObject);
begin
  SetFormFont(self, defaultFont, defaultFontSize);
end;

end.
