//-----------------------------------------------------------------------------------------
//| id | ff / 64   | 02 | player=0 / comp = 1 |  elf = 0 / ud = 1  | slotnr  | XX | 01 | 64 |
// -----------------------------------------------------------------------------------------
//                    |
//                    +-- 00 = open, 01 = closed, 02 = used

unit GameData;

interface

uses classes, windows, sysutils, wc3types;

type

  TSlotData = array[0..8] of byte;

  TRatio = record
    id: integer;
    tr: string[30]; // team ratio e.g. 4*v5v2
    team: integer;
  end;

  TGameData = class(Tobject)
  private
    fslots: integer;
    // each cardinal contains 32 bits, each bit states whether player-no=bit-no is in it
    fteams: array of cardinal;
    fratios: array of TRatio;
  public
    // index = slot number
    data: array[0..20] of TSlotData;
    left: array[0..20] of boolean;
    lostConnection: array[0..20] of boolean;
    rating: array[0..20] of integer;
    // index = id
    names: array[0..20] of string;
    freeSlotsDefaultValue: string;
    constructor Create;
    destructor Destroy; override;
    procedure ScanPacket(p: pointer);
    procedure SetName(i: integer; name: string);
    procedure SetTeamRatiosFirst;
    procedure SetTeamRatiosAfterLeaver;
    procedure SetLeft(id: integer);
    procedure SetOwnName;
    procedure Clear;
    function GetPlayersInTeam(t: integer): integer;
    function GetPlayerDistribution: string;
    function GetTeamRatioOfPlayer(id: integer): string; overload;
    function GetTeamRatioOfPlayer(n: string): string; overload;
    function GetTeam(id: integer): integer;
    function GetSlotByID(id: Byte): Integer; 
    function InGame(id: integer): boolean; overload;
    function InGame(nick: String): boolean; overload;
    function GetFreeSlots: string;
    function GetHumanPlayerSlotCount: Integer;
    // Properties
    property slots: integer read fslots;
  end;

implementation

uses unit1;

{ TGameData }

procedure TGameData.Clear;
var
  i: integer;
begin
  fslots := 0;
  for i := 0 to high(names) do
    names[i] := '';
  for i := 0 to high(left) do
    left[i] := false;
  for i := 0 to high(lostConnection) do
    lostConnection[i] := false;
  for i := 0 to high(rating) do
    rating[i] := 0;
  setlength(fratios, 0);
end;

constructor TGameData.Create;
begin
  Clear;
end;

destructor TGameData.Destroy;
begin
  inherited;
end;

function TGameData.GetPlayerDistribution: string;
var
  i: integer;
begin
  result := '';
  if high(fteams) = -1 then
    exit;
  result := inttostr(getplayersinteam(0));
  for i := 1 to high(fteams) do
    result := result + 'v' + inttostr(getplayersinteam(i));
end;

function TGameData.GetPlayersInTeam(t: integer): integer;
var
  i: integer;
  d: cardinal;
begin
  result := -1;
  if t > high(fteams) then
    exit;
  result := 0;
  d := fteams[t];
  for i := 0 to 31 do
  begin
    if odd(d) then
      inc(result);
    d := d shr 1;
  end;
end;

function TGameData.GetTeamRatioOfPlayer(id: integer): string;
var
  i: integer;
begin
  result := '';
  for i := 0 to high(fratios) do
    if fratios[i].id = id then
    begin
      result := fratios[i].tr;
      break;
    end;
end;

function TGameData.GetTeamRatioOfPlayer(n: string): string;
var
  i: integer;
  id: integer;
begin
  id := -1;
  for i := 0 to high(names) do
    if lowercase(names[i]) = lowercase(n) then
    begin
      id := i;
      break;
    end;
  if id > -1 then
    result := GetTeamRatioOfPlayer(id)
  else
    result := 'n/a';
end;

function TGameData.InGame(id: integer): boolean;
var
  i: integer;
begin
  result := false;
  for i := 0 to high(fteams) do
    result := result or (odd(fteams[i] shr id));
end;

procedure TGameData.SetOwnName;
var
  j: byte;
  i: integer;
begin
  for i := 0 to slots - 1 do
  begin
    j := Data[i][0];
    if (data[i][3] = 0) and (data[i][1] <> $FF) and (names[j] = '') then
      names[j] := '<self>';
  end;
end;

procedure TGameData.ScanPacket(p: pointer);
type
  bytearray = array[0..16000] of byte;
var
  b: ^bytearray;
  i, j: integer;
  teamcount: Integer;
begin
  b := p;
  // Only allow changes to slotnumber if 0
  if fslots = 0 then
    fslots := b^[6];
  // Filter errorneous Slot Information Packets
  if fslots <> b^[6] then
    exit;
  for i := 0 to slots - 1 do
    copymemory(@data[i], @b^[7 + i * 9], 9);
  // get teamcount without spectators
  teamcount := 0;
  for i := 0 to fslots - 1 do
    if (data[i][4] <> 12) and ((data[i][4] + 1) > teamcount) then
      teamcount := data[i][4] + 1;
  setlength(fteams, teamcount); // highstest team id + 1
  // Read teams and amount of players in each team
  for j := 0 to high(fteams) do
  begin
    fteams[j] := 0;
    for i := 0 to fslots - 1 do
    begin
      // human player and team correct?
      if (data[i][0] <> 0) and (data[i][4] = j) then
        fteams[j] := fteams[j] or (1 shl data[i][0]);
    end;
  end;
  // set own player name
  SetOwnName;
end;

procedure TGameData.SetLeft(id: integer);
var
  i: integer;
begin
  for i := 0 to high(fteams) do
    fteams[i] := fteams[i] and not (1 shl id);
end;

procedure TGameData.SetName(i: integer; name: string);
begin
  names[i] := name;
end;

procedure TGameData.SetTeamRatiosAfterLeaver;
var
  i: integer;
begin
  for i := 0 to high(fratios) do
    with fratios[i] do
      if InGame(id) then
      begin
        tr := GetPlayerDistribution;
        insert('*', tr, 2 * (team + 1));
      end;
end;

procedure TGameData.SetTeamRatiosFirst;
var
  i: integer;
  playercount: integer;
  x, y: integer;
begin
  playercount := 0;
  for i := 0 to high(fteams) do
    inc(playercount, GetPlayersInTeam(i));
  setlength(fratios, playercount);
  i := 0;
  x := 0;
  y := 0;
  while i < playercount do
  begin
    while (x < 32) and not odd(fteams[y] shr x) do
      inc(x); // player in this team?
    if x = 32 then
    begin
      inc(y);
      x := 0;
    end
    else
    begin
      with fratios[i] do
      begin
        id := x;
        team := y;
        tr := GetPlayerDistribution;
        insert('*', tr, (y + 1) * 2);
        inc(i);
        inc(x);
      end;
    end;
  end;
end;

function TGameData.GetTeam(id: integer): integer;
var
  i: integer;
begin
  result := -1;
  for i := 0 to slots - 1 do
    if data[i][0] = id then
    begin
      result := data[i][4];
      exit;
    end;
end;

function TGameData.GetFreeSlots: string;
var
  i: integer;
  n: integer;
begin
  if gamestate = gsChat then
    result := '$freeslots'
  else begin
    n := 0;
    for i := 0 to slots - 1 do
      if data[i][2] = 0 then
        inc(n);
    if slots = 0 then
      result := freeSlotsDefaultValue
    else
      result := inttostr(n);
  end;
end;

function TGameData.InGame(nick: String): boolean;
var
  i: Integer;
  b: Integer;
begin
  result := false;
  for i := 0 to slots - 1 do
  if (data[i][2] = 2) and (data[i][3] = 0) then begin
    b := data[i][0];
    if lowercase(nick)=lowercase(names[b]) then begin
      result := true;
      exit;
    end;
  end;
end;

function TGameData.GetSlotByID(id: Byte): Integer;
var
  i: Integer;
begin
  for i := 0 to slots - 1 do begin
    if data[i][0] = id then begin
      result := i;
      exit;
    end;
  end;
  result := 0;
end;

function TGameData.GetHumanPlayerSlotCount: Integer;
var
  i: integer;
  n: integer;
begin
  n := 0;
  for i := 0 to slots - 1 do
    if data[i][3] = 0 then
      inc(n);
  result := n;
end;

end.
