unit History;

interface

uses Classes, Windows;

type
  THistory = class(TStrings)
  private
    FTimeout: LongWord;
    FAppend: Boolean;
    FMaxChar: Integer;
  public
    constructor Create(millisecs: Cardinal; append: boolean; maxChar: integer);
    destructor Destroy; override;
    function Add(const s: String): Integer; override;
    procedure Clear; override;
    function Clean: boolean;
  end;

  TTimeCapsule = class(TObject)
  public
    Time : LongWord;
  end;

implementation

{ THistory }

function THistory.Add(const s: String): Integer;
var
  t: TTimeCapsule;
begin
  t := TTimeCapsule.Create;
  t.Time := GetTickCount;
  if FAppend then
    result := AddObject(s,t)
  else
  begin
    InsertObject(0,s,t);
    result := 0;
  end;
  Clean;
end;

// returns if there were changes
function THistory.Clean: boolean;
var
  i: Integer;
  t: TTimeCapsule;
begin
  i := 0;
  result := false;
  // remove older than timeout
  while i < Count do
  begin
    t := TTimeCapsule(Objects[i]);
    if GetTickCount - t.Time > FTimeOut then
    begin
      Objects[i].Free;
      Delete(i);
      result := true;
    end
    else
      inc(i);
  end;
  // remove longer than maxChar
  while Length(Text) > FMaxChar do
  begin
    result := true;
    if FAppend then
      Delete(0)
    else
      Delete(Count - 1);
  end;
end;

procedure THistory.Clear;
var
  i: Integer;
begin
  for i := 0 to Count-1 do
    Objects[i].Free;
  inherited;
end;

constructor THistory.Create;
begin
  FTimeOut := millisecs;
  FAppend := append;
  FMaxChar := maxChar;
end;

destructor THistory.Destroy;
begin
  inherited;
  Free;
end;



end.

