unit PhraseKeeper;

interface

uses
  Classes, SyncObjs, Windows, Messages, StrUtils, IniFile, WC3Types;

const
  ENDL = #$FE;

type
  TPhraseKeeper = class(TThread)
  private
    // for phrase delay calculation
    lasttick: dword;
    sentbytes: dword;
    // Phrase storage
    p: TList;
    lock: TCriticalSection;
    sem: THandle;
    abort: Boolean;
    lastBannedList: TStringList;
    { Private-Deklarationen }
  protected
    procedure Execute; override;
    function RequiredDelay(bytes: dword): word;
  public
    hWaitSemaphore: THandle;
    procedure SendStringToWC3(s: string);
    procedure Init;
    procedure Stop;
    procedure Add(s: string);
    procedure Clear;
    procedure TransferLastBannedList;
  end;

implementation

uses Unit1;

function TPhraseKeeper.RequiredDelay(bytes: dword): word;
const
  perpacket: dword = 200;
  msperbyte: dword = 50;
  maxbytes: dword = 600;
var
  tick: dword;
begin
  tick := GetTickCount();
  if (lasttick - tick > sentbytes * msperbyte) then
    sentbytes := 0
  else
    sentbytes := sentbytes - ((lasttick - tick) div msperbyte);
  lasttick := tick;
  if (sentbytes + perpacket + bytes > maxbytes) then
  begin
    result := (sentbytes + perpacket + bytes - maxbytes) * msperbyte;
    exit;
  end;
  sentbytes := sentbytes + perpacket + bytes;
  result := 0;
end;

procedure TPhraseKeeper.Add(s: string);
var
  t: TStringlist;
begin
  if s = '' then exit;
  t := Tstringlist.Create;
  t.Text := s + ENDL;
  lock.Enter;
  p.Add(t);
  lock.Leave;
  ReleaseSemaphore(sem, 1, nil);
end;

procedure TPhraseKeeper.Execute;
var
  s: string;
  t: TStringlist;
begin
  while not Terminated do
  begin
    WaitForSingleObject(sem, 500);
    if not Terminated then begin
      t := nil;
      lock.Enter;
      if p.Count > 0 then
      begin
        abort := false;
        t := p[0];
        p.Delete(0);
      end;
      lock.Leave;
      if t <> nil then
      begin
        s := t.Text;
        t.Free;
        SendStringToWC3(Copy(s, 1, Length(s) - Pos(ENDL, ReverseString(s))));
      end;
    end;
  end;
  CloseHandle(sem);
  Clear;
  P.Free;
  Lock.Free;
  lastBannedList.Free;
  CloseHandle(hWaitSemaphore);
end;

procedure TPhraseKeeper.Init;
begin
  p := TList.create;
  lock := TCriticalSection.Create;
  lasttick := 0;
  sentbytes := 0;
  sem := CreateSemaphore(nil, 0, 100, nil);
  lastBannedList := TStringList.Create;
  hWaitSemaphore := CreateSemaphore(nil, 0, 1, nil);
end;

procedure TPhraseKeeper.SendStringToWC3(s: string);
var
  h: Thandle;
  i, j: integer;
  bytes: dword;
  wait: boolean;
  vk: word;
begin
  h := findwindow('Warcraft III', 'Warcraft III');
  if h = 0 then
    exit;
  wait := true;
  i := 0;
  while i <= length(s) do
  begin
    if abort then break;
    if wait then
    begin
      bytes := 0;
      for j := i to length(s) do
        if s[j] = #13 then
          break
        else
          inc(bytes);
      if (GameState = gsChat) or (s[1] = '/') then
        Sleep(RequiredDelay(bytes));
      wait := false;
    end;
    if s[i] = '~' then
    begin
      sleep(500)
    end
    else if s[i] = #13 then
    begin
      postmessage(h, wm_keydown, vk_return, 0);
      postmessage(h, wm_keyup, vk_return, 0);
      wait := true;
    end
    else if s[i] = #255 then
    begin
      inc(i);
      if s[i] = #1 then
      begin
        postmessage(h, wm_keydown, vk_shift, 0);
        postmessage(h, wm_keydown, vk_return, 0);
        postmessage(h, wm_keyup, vk_return, 0);
        postmessage(h, wm_keyup, vk_shift, 0);
      end else
      if s[i] = #2 then
        insert(ct.GetText, s, i + 1)
      else
      if s[i] = #3 then
      begin
        Synchronize(TransferLastBannedList);
        if lastBannedList.Count > 0 then
          insert(lastbannedList[lastBannedList.Count - 1], s, i + 1);
      end else
      if s[i] = #4 then
      begin
        Synchronize(TransferLastBannedList);
        if lastBannedList.Count > 0 then
          insert(join(lastBannedList, ' '), s, i + 1);
      end else
      if s[i] = #5 then
      begin
        WaitForSingleObject(hWaitSemaphore, 10000);
      end else
      if (s[i] in [#240..#249]) then
      begin
        vk := (vk_numpad0 + ord(s[i]) - 240);
        postmessage(h, wm_keydown, vk, 0);
        postmessage(h, wm_keyup, vk, 0);
      end else
      if s[i] = #255 then
        postmessage(h, wm_char, ord(s[i]), 0);
    end else
      postmessage(h, wm_char, ord(s[i]), 0);
    inc(i);
  end;
end;

procedure TPhraseKeeper.Clear;
begin
  Lock.Enter;
  try
    while p.Count>0 do
    begin
      TStringList(p[0]).Free;
      p.Delete(0);
    end;
    abort := true;
  finally
    Lock.Leave;
  end;
end;

procedure TPhraseKeeper.Stop;
begin
end;

procedure TPhraseKeeper.TransferLastBannedList;
begin
  lastBannedList.Assign(unit1.lastBannedList);
end;

end.
