unit Pinger;

interface

uses Classes, Sysutils, SyncObjs, Forms, Windows;

type
  TPinger = class(TObject)
  private
    workers: integer;
    busyflag: boolean;
    cs: TCriticalSection;
  public
    jobs: TList;
    Done: procedure of object;
    constructor Create;
    procedure JobDone;
    procedure ClearJobs;
    function AddJob(name: string; ip: longword): integer;
    procedure PingAll;
    destructor Destroy; override;
    function Busy: boolean;
  end;

implementation

uses PingThread;

{ TPinger }

function TPinger.AddJob(name: string; ip: longword): integer;
var
  p: PPingJob;
begin
  result := -1;
  if length(name) > 29 then
    exit;
  if busyflag then
    exit;
  new(p);
  p^.name := name;
  p^.ip := ip;
  p^.ping := 0;
  result := jobs.add(p);
end;

function TPinger.Busy: boolean;
begin
  result := busyflag;
end;

procedure TPinger.ClearJobs;
begin
  while jobs.Count > 0 do
  begin
    dispose(jobs[0]);
    jobs.Delete(0);
  end;
  busyflag := false;
end;

constructor TPinger.Create;
begin
  jobs := Tlist.Create;
  cs := TCriticalSection.Create;
end;

destructor TPinger.Destroy;
begin
  ClearJobs;
  jobs.Free;
  cs.Free;
end;

procedure TPinger.JobDone;
begin
  Dec(workers);
  if (workers = 0) and Assigned(Done) then
    Done;
end;

procedure TPinger.PingAll;
var
  i: integer;
  t: TPingThread;
begin
  busyflag := true;
  workers := jobs.count;
  for i := 0 to jobs.count - 1 do
  begin
    t := tPingThread.Create(true);
    t.Priority := tpTimeCritical;
    t.job := jobs[i];
    t.parent := self;
    t.FreeOnTerminate := true;
    t.Resume;
  end;
  if (workers = 0) and (Assigned(Done)) then
    Done;  
end;

end.
