unit PlayerInfo;

interface

uses classes, windows, sysutils;

const
  VERTAG = '1.2';

type

  TVerTag = string[3];

  PBanInfo = ^TBanInfo;
  TBanInfo = record
    info: String;
    ban: String;
    onlineban: String;
  end;

  PPlayerInfo = ^TPlayerInfo;
  TPlayerInfo = record
    Nick: string[50];
    LastSeen: TDateTime;
    Whispered: Boolean;
    IP: DWORD;
    lastPing: Integer;
    banInfo: PBanInfo;
  end;

  TPlayerList = class(TObject)
  private
    Entries: TList;
  public
    constructor Create;
    destructor Destroy; override;
    procedure Add(Nick: string; IP: DWORD; var sameIP: TList);
    procedure SetNick(p: PPlayerInfo; nick: string);
    procedure Clear;
    function  GetIP(Nick: string): DWORD;
    procedure List(var l: TList);
    procedure SaveToFile(f: string);
    procedure LoadFromFile(f: string);
    // whispered state
    procedure SetWhispered(Nick: string);
    function  GetWhispered(Nick: string): boolean;
    procedure ClearWhispered;
    // baninfo
    function  GetBanInfo(Nick: String): PBanInfo;
    // playerinfo
    function  GetPlayerInfo(Nick: String): PPlayerInfo;
    // ping
    procedure SetPing(Nick: String; ping: Integer);
  end;

implementation

{ TPlayerList }

procedure TPlayerList.Add(Nick: string; IP: DWORD; var sameIP: TList);
var
  i: integer;
  p: PPlayerInfo;
  replaced: boolean;
begin
  i := 0;
  while i < entries.Count do
  begin
    p := PPlayerInfo(Entries[i]);
    if (now - p^.LastSeen) > 1 then
    begin
      Dispose(PPlayerInfo(Entries[i])^.banInfo);
      Dispose(Entries[i]);
      Entries.Delete(i);
    end else inc(i);
  end;
  replaced := false;
  for i := Entries.Count - 1 downto 0 do
  begin
    p := PPlayerInfo(Entries[i]);
    if lowercase(p^.Nick) = lowercase(Nick) then
    begin
      SetNick(p, nick);
      p^.LastSeen := Now;
      if ip <> 0 then
        p^.IP := IP; // keep old ip in case it was known but not now
      replaced := true;
      Entries.Delete(i);
      Entries.Add(p);
      break;
    end;
  end;
  if not replaced then
  begin
    New(p);
    SetNick(p, nick);
    p^.LastSeen := Now;
    p^.IP := IP;
    p^.lastPing := 0;
    new(p^.baninfo);
    p^.baninfo^.info := '';
    p^.baninfo^.onlineban := '';
    p^.baninfo^.ban := '';
    Entries.Add(p);
  end;
  p^.Whispered := false;
  if Assigned(sameIP) then
    for i := Entries.Count - 1 downto 0 do
    begin
      p := Entries[i];
      if (lowercase(p^.Nick) <> lowercase(Nick)) and (IP = p^.IP) then
        sameIP.Add(p);
    end;
end;

procedure TPlayerList.Clear;
var
  i: integer;
  p: PPlayerInfo;
begin
  for i := Entries.Count - 1 downto 0 do
  begin
    p := entries[i];
    Dispose(p^.baninfo);
    Dispose(p);
  end;
  Entries.Clear;
end;

procedure TPlayerList.ClearWhispered;
var
  p: PPlayerInfo;
  i: Integer;
begin
  for i := Entries.Count - 1 downto 0 do
  begin
    p := entries[i];
    p^.Whispered := false;
  end;
end;

constructor TPlayerList.Create;
begin
  Entries := TList.Create;
end;

destructor TPlayerList.Destroy;
begin
  Clear;
  Entries.Free;
end;

function TPlayerList.GetBanInfo(Nick: String): PBanInfo;
var
  i: integer;
  p: PPlayerInfo;
begin
  result := nil;
  for i := Entries.Count - 1 downto 0 do
  begin
    p := Entries[i];
    if lowercase(nick) = lowercase(p^.Nick) then
    begin
      result := p^.baninfo;
      break;
    end;
  end;
end;

function TPlayerList.GetIP(Nick: string): DWORD;
var
  i: integer;
  p: PPlayerInfo;
begin
  result := 0;
  for i := Entries.Count - 1 downto 0 do
  begin
    p := Entries[i];
    if lowercase(nick) = lowercase(p^.Nick) then
    begin
      result := p^.IP;
      break;
    end;
  end;
end;

function TPlayerList.GetPlayerInfo(Nick: String): PPlayerInfo;
var
  i: integer;
  p: PPlayerInfo;
begin
  result := nil;
  for i := Entries.Count - 1 downto 0 do
  begin
    p := Entries[i];
    if lowercase(nick) = lowercase(p^.Nick) then
    begin
      result := p;
      break;
    end;
  end;
end;

function TPlayerList.GetWhispered(Nick: string): boolean;
var
  i: integer;
  p: PPlayerInfo;
begin
  result := false;
  for i := Entries.Count - 1 downto 0 do
  begin
    p := Entries[i];
    if lowercase(nick) = lowercase(p^.Nick) then
    if p^.Whispered then begin
      result := true;
      break;
    end;
  end;
end;

procedure TPlayerList.List(var l: TList);
begin
  l.Assign(entries);
end;

procedure TPlayerList.LoadFromFile(f: string);
var
  s: TFileStream;
  p: PPlayerInfo;
  v: TVerTag;
begin
  Clear;
  s := TFileStream.Create(f, fmOpenRead);
  s.Read(v, SizeOf(Tvertag));
  if vertag = v then
    while not (s.Position = s.Size) do
    begin
      new(p);
      s.Read(p^, sizeof(TPlayerInfo));
      new(p^.baninfo);
      p^.baninfo^.info := '';
      p^.baninfo^.onlineban := '';
      p^.baninfo^.ban := '';
      p^.lastPing := 0;
      Entries.Add(p);
    end;
  s.free;
end;

procedure TPlayerList.SaveToFile(f: string);
var
  s: TFileStream;
  p: PPlayerInfo;
  i: integer;
  v: tvertag;
begin
  s := TFileStream.Create(f, fmCreate);
  v := vertag;
  s.Write(v, sizeof(tvertag));
  for i := 0 to Entries.count - 1 do
  begin
    p := entries[i];
    s.Write(p^, sizeof(TplayerInfo));
  end;
  s.free;
end;

procedure TPlayerList.SetNick(p: PPlayerInfo; nick: string);
begin
  // in order to check size
  p^.Nick := Copy(nick, 1, 49);
end;

procedure TPlayerList.SetPing(Nick: String; ping: Integer);
var
  i: integer;
  p: PPlayerInfo;
begin
  for i := Entries.Count - 1 downto 0 do
  begin
    p := Entries[i];
    if lowercase(nick) = lowercase(p^.Nick) then
    begin
      p^.lastPing := ping; 
      break;
    end;
  end;
end;

procedure TPlayerList.SetWhispered(Nick: string);
var
  i: integer;
  p: PPlayerInfo;
begin
  for i := Entries.Count - 1 downto 0 do
  begin
    p := Entries[i];
    if lowercase(nick) = lowercase(p^.Nick) then
    begin
      p^.Whispered := True;
      break;
    end;
  end;
end;

end.
