unit ServerList;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ComCtrls, Buttons, inifile, VirtualTrees, Wc3Types,
  IdHTTP, ProxyInfo, IdComponent, IdCompressorZLibEx;

type

  TServerList = class(TObject)
  private
    FParent: TPageControl;
    FListType: TListType;
    FTabSheet: TTabSheet;
    FMemo: TMemo;
    FView: TVirtualStringTree;
    FTitle: string;
    FRealm: string;
    FUrl: string;
    FOnError: TNotifyEvent;
    FReady: Boolean;
    FError: Boolean;
    FBackup: Boolean;
    FLoader: TThread;
    FUsingBackup: Boolean;
    FIgnoreErrors: Boolean;
    FBackupDate: TDateTime;
  protected
    function AddEntry(Nickname, Comment: string): PBanBuddiesBanlist;
    procedure LockView;
    procedure UnlockView;
    procedure ShowError(msg: string);
    procedure ShutdownLoader;
    procedure SetUsingBackup;
    procedure DoOnError;
  public
    constructor Create(Parent: TPageControl; ErrorMemo: TMemo;
      ListType: TListType; Title: string; Realm: string; Url: string; IgnoreErrors: Boolean);
    destructor Destroy; override;
    procedure Load;
    procedure ShowProgess(kB: Integer);
    procedure SetNewURL(URL: String);
    function IsListed(Nickname: string): PBanBuddiesBanlist;
    procedure SetListCount;
    property ListType: TListType read FListType;
    property Ready: Boolean read FReady write FReady;
    property Title: string read FTitle;
    property Tabsheet: TTabSheet read FTabSheet;
    property ErrorOccured: Boolean read FError;
    property OnError: TNotifyEvent read FOnError write FOnError;
    property Backup: Boolean read FBackup write FBackup;
    property UsingBackup: Boolean read FUsingBackup write FUsingBackup;
    property IgnoreErrors: Boolean read FIgnoreErrors write FIgnoreErrors;
  end;

  TListLoader = class(TThread)
  private
    kB: Integer;
  protected
    procedure PasteEntries;
    procedure ShowError;
    procedure ShowProgress;
    procedure Execute; override;
    procedure IdHTTP1Work(ASender: TObject; AWorkMode: TWorkMode;
      AWorkCount: Integer);
    procedure IdHTTP1Redirect(Sender: TObject; var dest: String;
      var NumRedirect: Integer; var Handled: Boolean; var VMethod: TIdHTTPMethod);
    procedure PermanentRedirect;
  public
    ServerList: TServerList;
    Data: TStringlist;
    newURL, URL: string;
    ErrorMessage: string;
  end;

var
  PageCount: Integer;
  ErrorCount: Integer;
  ErroneousNicks: String;

implementation

uses Unit1;

{ TServerList }

function GetFileModifyDate(FileName: string): TDateTime;
var
  h: THandle;
  Struct: TOFSTRUCT;
  lastwrite: Integer;
begin
  result := 0;
  h := OpenFile(PChar(FileName), Struct, OF_SHARE_DENY_NONE);
  try
    if h <> HFILE_ERROR then
    begin
      lastwrite := FileGetDate(h);
      Result    := FileDateToDateTime(lastwrite);
    end;
  finally
    CloseHandle(h);
  end;
end;

function TServerList.AddEntry(Nickname, Comment: string): PBanBuddiesBanlist;
var
  p: PBanBuddiesBanlist;
  n: PVirtualNode;
begin
  n := FView.AddChild(nil);
  p := FView.GetNodeData(n);
  FView.ValidateNode(n, false);
  p^.nickname := Nickname;
  p^.comment := Comment;
  p^.source := Title;
  p^.imageindex := -1;
  result := p;
end;

constructor TServerlist.Create(Parent: TPageControl; ErrorMemo: TMemo;
  ListType: TListType; Title: string; Realm: string; Url: string; IgnoreErrors: Boolean);
begin
  FParent := Parent;
  FListType := ListType;
  FTitle := Title;
  FRealm := Realm;
  FUrl := Url;
  FMemo := ErrorMemo;
  FIgnoreErrors := IgnoreErrors;
  FTabSheet := TTabSheet.Create(FParent);
  with FTabSheet do
  begin
    Caption := FTitle;
    Parent := FParent;
    Name := 'pgServerList' + IntToStr(PageCount);
    if ListType = ltBanlist then
      ImageIndex := 4
    else
      ImageIndex := 5;
    PageControl := FParent;
  end;
  FView := TVirtualStringTree.Create(FTabSheet);
  with FView do
  begin
    Name := 'vstServerListView' + IntToStr(PageCount);
    Parent := FTabSheet;
    Left := 0;
    Top := 0;
    Width := 752;
    Height := 390;
    Align := alClient;
    Images := WC3Banlist_Main.ilBanlist;
    //    PopupMenu := popBuddy;
    TabOrder := 0;
    with Header.Columns.Add do
    begin
      Alignment := taCenter;
      Options := [coAllowClick, coDraggable, coEnabled, coParentBidiMode,
        coParentColor, coShowDropMark, coVisible];
      Position := 0;
      Width := 150;
      Text := 'Nickname';
    end;
    with Header.Columns.Add do
    begin
      Position := 1;
      Options := [coAllowClick, coDraggable, coEnabled, coParentBidiMode,
        coParentColor, coShowDropMark, coVisible];
      Width := FView.Width - 150;
      Text := 'Comment';
    end;
    Header.Options := [hoAutoResize, hoColumnResize, hoDrag, hoShowSortGlyphs,
      hoVisible];
    Header.AutoSizeIndex := 1;
    Header.Font.Charset := DEFAULT_CHARSET;
    Header.Font.Color := clWindowText;
    Header.Font.Height := -11;
    Header.Font.Name := defaultFont;
    Header.Font.Style := [];
    Header.SortColumn := 0;
    Header.Style := hsXPStyle;
    Font.Name := defaultFont;
    TreeOptions.AutoOptions := [toAutoDropExpand, toAutoScrollOnExpand,
      toAutoSort, toAutoTristateTracking, toAutoDeleteMovedNodes];
    TreeOptions.PaintOptions := [toShowButtons, toShowDropmark,
      toShowHorzGridLines, toShowVertGridLines, toThemeAware, toUseBlendedImages,
      toFullVertGridLines];
    TreeOptions.SelectionOptions := [toFullRowSelect, toMultiSelect,
      toRightClickSelect, toSimpleDrawSelection];
  end;
  Inc(PageCount);
  FView.NodeDataSize := SizeOf(TBanBuddiesBanlist);
  with WC3Banlist_Main do
  begin
    FView.OnGetText := vstBuddyGetText;
    FView.OnCompareNodes := vstBuddyCompareNodes;
    FView.OnHeaderClick := vstBuddyHeaderClick;
    FView.OnGetImageIndex := vstBuddyGetImageIndex;
    FView.OnFreeNode := vstBuddyFreeNode;
  end;
end;

destructor TServerList.Destroy;
begin
  ShutdownLoader;
  FReady := false;
  FreeAndNil(FView);
  FreeAndNil(FTabSheet);
  inherited;
end;

function TServerList.IsListed(Nickname: string): PBanBuddiesBanlist;
var
  n: PVirtualNode;
  p: PBanBuddiesBanlist;
begin
  n := FView.GetFirst;
  result := nil;
  while (result = nil) and (n <> nil) do
  begin
    p := FView.GetNodeData(n);
    if lowercase(Nickname) = lowercase(p^.nickname) then
      result := p;
    n := FView.GetNext(n);
  end;
end;

procedure TServerList.Load;
var Loader: TListLoader;
begin
  ShutDownLoader;
  FReady := False;
  FError := False;
  FUsingBackup := False;
  FView.Clear;
  Loader := TListLoader.Create(True);
  Loader.URL := FURL;
  Loader.ServerList := self;
  Loader.Resume;
  FLoader := Loader;
end;

procedure TServerList.LockView;
begin
  FView.BeginUpdate;
  FView.Clear;
end;

procedure TServerList.SetListCount;
begin
  FTabSheet.Caption := FTitle + ' (' + Inttostr(FView.TotalCount) + ')';
end;

procedure TServerList.ShowError(msg: string);
var
  t: TStringlist;
  i: Integer;
begin
  FMemo.Lines.Add(FTitle +
    ' - download failed...');
  t := TStringlist.Create;
  try
    t.Text := msg;
    for i := 0 to t.Count - 1 do
      FMemo.Lines.Add('  ' + t[i]);
  finally
    t.Free;
  end;
  FError := True;
  FTabSheet.Caption := FTitle + ' (error)';
end;

procedure TServerList.ShowProgess(kB: Integer);
begin
  FTabSheet.Caption := FTitle + ' (' + Inttostr(kB) + ' kB)';
end;

procedure TServerList.SetUsingBackup;
begin
  FTabSheet.Caption := FTitle + ' (backup: ' + DateToStr(FBackupDate) + ')';
  FUsingBackup := True;
  WC3Banlist_Main.lvOnlineLists.Invalidate;
end;

procedure TServerList.ShutdownLoader;
begin
  if Assigned(FLoader) then
  begin
    FLoader.Terminate;
    FLoader.WaitFor;
    FreeAndNil(TListLoader(FLoader));
  end;
end;

procedure TServerList.UnlockView;
begin
  FView.EndUpdate;
end;

procedure TServerList.DoOnError;
begin
  FParent.ActivePage := FTabSheet;
  if Assigned(FOnError) then
    FOnError(self);
end;

procedure TServerList.SetNewURL(URL: String);
begin
  WC3Banlist_Main.SetNewURLforServerList(URL, self);
end;

{ TListLoader }

procedure TListLoader.Execute;
var
  idhttp1: TidHTTP;
  idCompressorZLibEx1: TIdCompressorZLibEx;
  proxy: string;
  port: Integer;
  backupUsed: boolean;
begin
  kB := 0;
  backupUsed := false;
  idhttp1 := TidHTTP.Create(ServerList.Tabsheet);
  idCompressorZLibEx1 := TIdCompressorZLibEx.Create;
  idhttp1.Compressor := idCompressorZLibEx1;
  idhttp1.OnWork := IdHTTP1Work;
  idhttp1.ConnectTimeout := 10000;
  idhttp1.ReadTimeout := 30000;
  idhttp1.Request.AcceptEncoding := 'gzip, deflate, ';
  idhttp1.HandleRedirects := True;
  idhttp1.OnRedirect := IdHTTP1Redirect;
  data := TStringlist.create;
  // Set Proxy
  if WC3Banlist_Main.chkproxy.Checked then
  begin
    getproxyserver('http', proxy, port);
    idhttp1.ProxyParams.ProxyServer := proxy;
    idhttp1.ProxyParams.ProxyPort := port;
  end
  else
  begin
    idhttp1.ProxyParams.ProxyServer := '';
    idhttp1.ProxyParams.ProxyPort := 0;
  end;
  try
    try
      if pos('?', URL) > 0 then
        data.Text := idhttp1.Get(URL + '&clientversion=' + HTTPEncode(CURRENT_VERSION))
      else
        data.Text := idhttp1.Get(URL + '?clientversion=' + HTTPEncode(CURRENT_VERSION));
      ErrorCount := 0;
      ErroneousNicks := '';
      Synchronize(PasteEntries);
      if not ServerList.IgnoreErrors and (ErrorCount > 0) then
        raise Exception.Create('The server list is corrupted. There are ' + IntToStr(ErrorCount) + ' malformed nicknames in this server list.' + ErroneousNicks);
      if ServerList.Backup then
      begin
        ForceDirectories(IniPath + 'backups');
        Data.SaveToFile(IniPath + 'backups\' + ServerList.FTitle);
      end;
    finally
      idhttp1.Free;
    end;
  except
    on e: Exception do
    begin
      ErrorMessage := e.Message;
      try
        if ServerList.Backup and
          FileExists(IniPath + 'backups\' + ServerList.FTitle) then
        begin
          ServerList.FBackupDate := GetFileModifyDate(IniPath + 'backups\' + ServerList.FTitle);
          Data.LoadFromFile(IniPath + 'backups\' + ServerList.FTitle);
          Synchronize(PasteEntries);
          ErrorMessage := ErrorMessage + cr + 'The backup will be used instead.';
          backUpUsed := true;
        end;
      except
        on e2: Exception do
        begin
          ErrorMessage := ErrorMessage + cr +
            'There also was an error while using the backup of the list ' +
            ServerList.FTitle +':' + cr + e2.Message; 
        end;
      end;
      if not Terminated then begin
        Synchronize(ShowError);
        if backupUsed then
          Synchronize(ServerList.SetUsingBackup);
        Synchronize(Serverlist.DoOnError);
      end;
    end;
  end;
  data.Free;
  idCompressorZLibEx1.Free;
end;

procedure TListLoader.IdHTTP1Redirect(Sender: TObject; var dest: String;
  var NumRedirect: Integer; var Handled: Boolean; var VMethod: TIdHTTPMethod);
begin
  if (sender as TidHTTP).ResponseCode = 301 then
  begin
    newURL := dest;
    Synchronize(PermanentRedirect);
  end;
end;

procedure TListLoader.IdHTTP1Work(ASender: TObject; AWorkMode: TWorkMode;
  AWorkCount: Integer);
begin
  if terminated and TIdHTTP(ASender).Connected then
    TIdHTTP(ASender).Disconnect;
  kB := AWorkCount shr 10;
  Synchronize(ShowProgress);
end;

procedure TListLoader.PasteEntries;
var
  i, p: Integer;
  a: string;
  Nickname, Comment: string;
begin
  if closing then exit;
  ServerList.LockView;
  try
    for i := 0 to Data.Count - 1 do
    begin
      a := Data[i];
      p := Pos(' ', a);
      if p = 0 then
      begin
        NickName := a;
        Comment := '';
      end
      else
      begin
        NickName := Copy(a, 1, p - 1);
        Delete(a, 1, p);
        Comment := a;
      end;
      if NickName <> '' then
      begin
        if not CheckNicknameSpelling(NickName) then
        begin
          inc(ErrorCount);
          ErroneousNicks := ErroneousNicks + #13#10 + NickName;
        end;
        ServerList.AddEntry(NickName, Comment);
      end;
    end;
  finally
    ServerList.UnlockView;
  end;
  Serverlist.SetListCount;
  ServerList.FReady := True;
  WC3Banlist_Main.lvOnlineLists.Invalidate;
end;

procedure TListLoader.PermanentRedirect;
begin
  ServerList.SetNewURL(newURL);
end;

procedure TListLoader.ShowError;
begin
  ServerList.ShowError(ErrorMessage);
end;

procedure TListLoader.ShowProgress;
begin
  ServerList.ShowProgess(kB);
end;

end.
