unit StringCache;

interface

uses Classes, Windows;

type
  TStringCache = class(TObject)
  private
    FStrings: TStringList;
    FTimeout: LongWord;
  public
    constructor Create(millisecs: Cardinal);
    function AddAndCheckInList(s: String): boolean;
    destructor Destroy; override;
    function Text: string;
  end;

implementation

{ TStringCache }

function TStringCache.AddAndCheckInList(s: String): boolean;
var
  i: Integer;
  t: Cardinal;
begin
  result := false;
  // remove older than timeout
  i := 0;
  while i < FStrings.Count do
  begin
    t := Cardinal(FStrings.Objects[i]);
    if GetTickCount - t > FTimeOut then
      FStrings.Delete(i)
    else
      inc(i);
  end;
  i := FStrings.IndexOf(s);
  // in list of objects younger than timeout?
  if i >= 0 then begin
    result:= true;
    // set new timestamp
    FStrings.Objects[i] := Pointer(GetTickCount);
  end else
    FStrings.AddObject(s, Pointer(GetTickCount));
end;

constructor TStringCache.Create;
begin
  FStrings := TStringlist.Create;
  FTimeOut := millisecs;
end;

destructor TStringCache.Destroy;
begin
  FStrings.Free;
  inherited;
end;



function TStringCache.Text: string;
begin
  result := FStrings.Text;
end;

end.
