unit UAutoRefresh;

interface

uses
  Classes, idTCPClient, Windows, SysUtils;

const
  BA_HEADER: String = #$F7#$1E;
  BA_STUFF:  String = #0#$E0#$17#$08#0#0#0;
  BA_STUFF2: String = #1#0#2#0#$17#$E0#$C0#$A8#1#1#0#0#0#0#0#0#0#0;

type
  TConnector = class(TThread)
  private
    Packet: String;
    Ip: String;
    Port: Word;
    Tcp: TIdTCPClient;
    fullGameSem: THandle;
  protected
    procedure Execute; override;
  public
    constructor Create(packet: String; ip: String; port: Word; fullGameSem: THandle);
    destructor Destroy; override;
  end;

  TAutoRefresher = class(TThread)
  private
    hSem: THandle;
    hProt: THandle;
    FInterval: Cardinal;
    FGameCode: String;
    FNickName: String;
    FIP: String;
    FPort: Word;
    FEnabled: Boolean;
    TimerReset: Boolean;
    Triggered: Boolean;
    LastRefresh: Cardinal;
    FreeSlots: Integer;
  protected
    procedure Execute; override;
    function GetInterval: Cardinal;
    function GetGameCode: String;
    function GetNickName: String;
    procedure SetInterval(Value: Cardinal);
    procedure SetGameCode(Value: String);
    procedure SetNickName(Value: String);
    procedure SetEnabled(Value: Boolean);
    function GetEnabled: Boolean;
    procedure GetFreeSlotCount;
  public
    constructor Create(IP: String; Port: Word);
    destructor Destroy; override;
    procedure Trigger;
    procedure ResetTimer;
    property Enabled: Boolean read GetEnabled write SetEnabled;
    property Interval: Cardinal read GetInterval write SetInterval;
    property GameCode: String read GetGameCode write SetGameCode;
    property NickName: String read GetNickName write SetNickName;
  end;

implementation

uses unit1;


{ TAutoRefresher }

constructor TAutoRefresher.Create(IP: String; Port: Word);
begin
  inherited Create(true);
  FInterval := 30000;
  FGameCode := '';
  FNickName := '|CFFFFF200|R';
  FIP := IP;
  FPort := Port;
  hSem := CreateSemaphore(nil, 0, 1, nil);
  hProt := CreateSemaphore(nil, 1, 1, nil);
  Enabled := False;
  TimerReset := False;
  Triggered := False;
  LastRefresh := 0;
  Resume;
end;

destructor TAutoRefresher.Destroy;
begin
  CloseHandle(hSem);
  CloseHandle(hProt);
  inherited;
end;

procedure TAutoRefresher.Execute;
var
  packet: String;
  i: Integer;
  asciigc, gc, name: String;
  size: SmallInt;
  gameNumber: Integer;
  connectors: TList;
  fullGameSem: Thandle;
begin
  while not terminated do
  begin
    WaitForSingleObject(hSem, Interval);
    // create packet data
    if not terminated then
    begin
      gc := '';
      asciigc := GameCode;
      if (asciigc <> '') and (Enabled or Triggered) and
        not TimerReset and (GetTickCount - LastRefresh > 5000) then
      begin
        gamenumber := 0;
        for i := 0 to length(asciigc) - 1  do gamenumber := gamenumber + StrToInt('$' + asciigc[i + 1]) shl (i*4);
        gc := '';
        for i := 0 to 3 do gc := gc + chr((gamenumber shr (i * 8)) and $FF);
        gc := gc + #0#0#0#0;
        size := 0;
        name := NickName;
        packet := BA_HEADER + Chr(size and $FF) + Chr(size shr 8) + gc + BA_STUFF + name + #0 + BA_STUFF2;
        size := length(packet);
        packet := BA_HEADER + Chr(size and $FF) + Chr(size shr 8) + gc + BA_STUFF + name + #0 + BA_STUFF2;
        // create connectors
        LastRefresh := GetTickCount;
        Connectors := TList.Create;
        fullGameSem := CreateSemaphore(nil, 0, 100, nil);
        for i := 1 to 13 do
        begin
          connectors.add(TConnector.Create(packet, FIp, FPort, fullGameSem));
          if WaitForSingleObject(fullGameSem, 100) <> WAIT_TIMEOUT then
            break;
        end;
        CloseHandle(fullGameSem);
        Connectors.Free;
      end;
      TimerReset := False;
      Triggered := False;
    end;
  end;
end;

function TAutoRefresher.GetEnabled: Boolean;
begin
  WaitForSingleObject(hProt, INFINITE);
  result := FEnabled;
  ReleaseSemaphore(hProt, 1, nil);
end;

procedure TAutoRefresher.GetFreeSlotCount;
begin
  try
    FreeSlots := StrToInt(gd.getfreeslots);
  except
    FreeSlots := 12;
  end;
end;

function TAutoRefresher.GetGameCode: String;
begin
  WaitForSingleObject(hProt, INFINITE);
  result := FGameCode;
  ReleaseSemaphore(hProt, 1, nil);
end;

function TAutoRefresher.GetInterval: Cardinal;
begin
  WaitForSingleObject(hProt, INFINITE);
  result := FInterval;
  ReleaseSemaphore(hProt, 1, nil);
end;

function TAutoRefresher.GetNickName: String;
begin
  WaitForSingleObject(hProt, INFINITE);
  result := FNickName;
  ReleaseSemaphore(hProt, 1, nil);
end;

procedure TAutoRefresher.ResetTimer;
begin
  TimerReset := true;
  Trigger;
end;

procedure TAutoRefresher.SetEnabled(Value: Boolean);
begin
  WaitForSingleObject(hProt, INFINITE);
  FEnabled := Value;
  ReleaseSemaphore(hProt, 1, nil);
end;

procedure TAutoRefresher.SetGameCode(Value: String);
begin
  WaitForSingleObject(hProt, INFINITE);
  FGameCode := Value;
  ReleaseSemaphore(hProt, 1, nil);
end;

procedure TAutoRefresher.SetInterval(Value: Cardinal);
begin
  WaitForSingleObject(hProt, INFINITE);
  FInterval := Value;
  ReleaseSemaphore(hProt, 1, nil);
end;

procedure TAutoRefresher.SetNickName(Value: String);
begin
  WaitForSingleObject(hProt, INFINITE);
  FNickName := Value;
  ReleaseSemaphore(hProt, 1, nil);
end;

procedure TAutoRefresher.Trigger;
begin
  Triggered := True;
  ReleaseSemaphore(hSem, 1, nil);
end;

{ TConnector }

constructor TConnector.Create(packet: String; ip: String; port: Word; fullGameSem: Thandle);
begin
  inherited Create(true);
  self.packet := packet;
  self.ip := ip;
  self.port := port;
  self.fullGameSem := fullGameSem;
  FreeOnTerminate := True;
  tcp := TIdTCPClient.Create;
  tcp.ConnectTimeout := 200;
  tcp.ReadTimeout := 200;
  Resume;
end;

destructor TConnector.Destroy;
begin
  inherited;
end;

procedure TConnector.Execute;
var
  c: Char;
begin
  try
    tcp.connect(ip, port);
    // only send login packet if game isn't full 
    if WaitForSingleObject(fullGameSem, 0) = WAIT_TIMEOUT then
    begin
      tcp.Socket.WriteBufferOpen;
      tcp.Socket.Write(packet);
      tcp.Socket.WriteBufferFlush;
      tcp.Socket.ReadChar;
      c := tcp.Socket.ReadChar;
      // connection refused?
      if c = #5 then
        ReleaseSemaphore(fullGameSem, 100, nil)
      else
      // wait for max 3 sec
        WaitForSingleObject(fullGameSem, 3000);
    end;
    tcp.Socket.CloseGracefully;
  except
  end;
  tcp.Free;
end;

end.
