unit UCountryCache;

interface

uses
  Classes, Windows, Sysutils, Inifile;

type
  TFrom = record
    ip: string;
    country: string;
  end;

  PFrom = ^TFrom;

  TCountryCache = class(TObject)
  private
    FLong: TList;
    hSem: THandle;
    procedure Lock;
    procedure Unlock;
  public
    constructor Create;
    destructor Destroy; override;
    procedure Clear;
    function Check(ip: string): string;
    procedure Add(ip: string; country: string);
  end;

implementation

{ TCountyCache }

procedure TCountryCache.Add(ip, country: string);
var
  p: PFrom;
  l: TList;
begin
  Lock;
  l := FLong;
  new(p);
  p^.ip := ip;
  p^.country := country;
  l.Add(p);
  Unlock;
end;

function TCountryCache.Check(ip: string): string;
var
  i: integer;
  p: PFrom;
  l: TList;
begin
  Lock;
  result := '';
  l := FLong;
  for i := 0 to l.Count - 1 do
  begin
    p := l[i];
    if p^.ip = ip then
    begin
      result := p^.country;
      break;
    end;
  end;
  Unlock;
end;

procedure TCountryCache.Clear;
var
  p: PFrom;
  l: TList;
begin
  Lock;
  l := FLong;
  while l.Count > 0 do
  begin
    p := l[0];
    Dispose(p);
    l.Delete(0);
  end;
  Unlock;
end;

constructor TCountryCache.Create;
begin
  FLong := TList.Create;
  hSem := CreateSemaphore(nil, 1, 1, nil);
end;

destructor TCountryCache.Destroy;
begin
  Clear;
  FLong.Free;
  CloseHandle(hSem);
  inherited;
end;

procedure TCountryCache.Lock;
begin
  WaitForSingleObject(hSem, INFINITE);
end;

procedure TCountryCache.Unlock;
begin
  ReleaseSemaphore(hSem, 1, nil);
end;

end.
