unit UIPToCountry;

interface

uses
  Classes, UCountryCache, SysUtils, IdHTTP, ProxyInfo, IniFile, RegExpr,
  Windows;

type
  TIPToCountry = class(TThread)
  private
    CountryCache: TCountryCache;
    Failed: Boolean;
    Text: String;
    IPList: TStringList;
    NameList: TStringlist;
    HTTP: TIdHTTP;
  protected
    procedure Execute; override;
  public
    constructor Create(aCountryCache: TCountryCache);
    destructor Destroy; override;
    procedure ResolveSingle(aIP, aName: String);
    procedure ResolveList(aIPList, aNameList: TStringList);
    procedure PresentResult;
  end;

implementation

uses Unit1;

{ IPToCountry }

constructor TIPToCountry.Create(aCountryCache: TCountryCache);
var
  proxy: String;
  port: Integer;
begin
  inherited Create(true);
  CountryCache := aCountryCache;
  IPList := TStringList.Create;
  NameList := TStringList.Create;
  Failed := False;
  HTTP := TIdHTTP.Create(nil);
  if WC3Banlist_Main.chkProxy.checked then
  begin
    GetProxyServer('http', proxy, port);
    HTTP.ProxyParams.ProxyServer := proxy;
    HTTP.ProxyParams.ProxyPort := port;
  end;
  HTTP.ReadTimeout := 3000;
  FreeOnTerminate := True;
end;

destructor TIPToCountry.Destroy;
begin
  IPList.Free;
  NameList.Free;
  HTTP.Free;
  inherited;
end;

procedure TIPToCountry.Execute;
var
  country: String;
  countries, newIPs, newCountries, results: TStringList;
  i, j: Integer;
  reg: TRegExpr;
begin
  if (IPList.Count = 1) then
  begin
    country := CountryCache.Check(IPList[0]);
    if country = '' then
    begin
      try
        country := HTTP.Get('http://wc3banlist.de/iptc.php?addr=' + IPList[0]);
        if country = '' then country := ' ()';
        CountryCache.Add(IPList[0], country);
      except
        on e: Exception do
        begin
          Failed := True;
          Text := e.Message;
          Text := StringReplace(Text, #13#10, ' ', [rfReplaceAll]) +
            ' IP2Country failed. Check your proxy/firewall settings.';
        end;
      end;
    end;
    if not Failed then
    begin
      if country = ' ()' then
      begin
        if NameList[0] = 'The host' then
          NameList[0] := 'the host';
        Text := 'Could not find out ' + NameList[0] + '''s origin.';
      end
      else
        Text := NameList[0] + ' comes from ' + country + '.';
    end;
  end else
  if (IPList.Count > 1) then
  begin
    countries := TStringList.Create;
    newCountries := TStringlist.Create;
    newIPs := TStringList.Create;
    results := TStringList.Create;

    // read cache
    for i := 0 to IPList.Count - 1 do
      countries.Add(CountryCache.Check(IPList[i]));


    // create list of missing ips
    for i := 0 to countries.Count - 1 do
      if countries[i] = '' then
        newIPs.Add(IPList[i]);

    // retrieve missing ips
    try
      country := HTTP.Get('http://wc3banlist.de/iptc.php?addr=' + join(newIPs, ';'));
      if country <> '' then
        split(country, ';', newCountries);

      // add new results to cache
      for i := 0 to newCountries.Count - 1 do
        if newCountries[i] = '' then
          CountryCache.Add(newIPs[i], ' ()')
        else
          CountryCache.Add(newIPs[i], newCountries[i]);

      // use new results
      j := 0;
      for i := 0 to countries.Count - 1 do
        if countries[i] = '' then
        begin
          countries[i] := newCountries[j];
          inc(j);
        end;

      // create message text
      reg := TRegExpr.Create;
      reg.Expression := '\(.+\)';
      for i := 0 to countries.Count - 1 do
      begin
        if reg.Exec(countries[i]) then
          results.Add(NameList[i] + ': ' + reg.Match[0])
        else
          results.Add(NameList[i] + ': unknown');
      end;
      reg.Free;
      Text := Join(results, ', ');
      
    except
        on e: Exception do
        begin
          Failed := True;
          Text := e.Message;
          Text := StringReplace(Text, #13#10, ' ', [rfReplaceAll]) +
            ' IP2Country failed. Check your proxy/firewall settings.';
        end;
    end;

    newIPs.Free;
    newCountries.Free;
    countries.Free;
    results.Free;
  end;

  if Text <> '' then
    Synchronize(PresentResult);
end;

procedure TIPToCountry.PresentResult;
begin
  if Failed then
    WC3Banlist_Main.AddMessage(Text, ICON_ERROR)
  else
  begin
    if NameList.Count = 1 then
      WC3Banlist_Main.AddMessage(Text, ICON_WORLD, NameList[0])
    else
      WC3Banlist_Main.AddMessage(Text, ICON_WORLD);
  end;
  ReleaseSemaphore(phraseKeeper1.hWaitSemaphore, 1, nil);
  WC3Banlist_Main.Sound(SND_RESPONSE);
end;

procedure TIPToCountry.ResolveList(aIPList, aNameList: TStringList);
begin
  IPList.Assign(aIPList);
  NameList.Assign(aNameList);
  Resume;
end;

procedure TIPToCountry.ResolveSingle(aIP, aName: String);
begin
  IPList.Clear;
  IPList.Add(aIP);
  NameList.Clear;
  NameList.Add(aName);
  Resume;
end;

end.
