unit UMapInfo;

interface

uses
  sfmpqapi, RegExpr, Classes, SysUtils, Windows;

type
  TMapInfo = class(TObject)
  private
    function CountPlayers(text: PChar): integer;
    function GetColors(text: PChar): boolean;
    function GetTeams(text: PChar): boolean;
  public
    colors: array[0..11] of shortint;
    colorsOK: boolean;
    teams: array[0..11] of shortint;
    teamsOK: boolean;
    playerCount: Integer;
    constructor Create;
    destructor Destroy; Override;
    procedure Clear;
    function ReadMap(FileNameOfMap: String): boolean;
  end;

implementation

{ TMapInfo }

procedure TMapInfo.Clear;
begin
  FillChar(colors, sizeof(colors), -1);
  FillChar(teams, sizeof(teams), -1);
  playerCount := 0;
  teamsOK := false;
  colorsOK := false;
end;

function TMapInfo.CountPlayers(text: PChar): integer;
var
  r: TRegExpr;
  c: Integer;
begin
  r := TRegExpr.Create;
  c := 0;
  try
    r.InputString := text;
    r.Expression := '\s*SetPlayerController\s*\(\s*Player\s*\((\d*)\)\s*,\s*MAP_CONTROL_USER\s*\)';
    if r.Exec then
    begin
      inc(c);
      while r.ExecNext do inc(c);
    end;
  finally
    r.Free;
  end;
  result := c;
end;

constructor TMapInfo.Create;
begin
  Clear;
end;

destructor TMapInfo.Destroy;
begin

  inherited;
end;

function TMapInfo.GetColors(text: PChar): boolean;
var
  r: TRegExpr;
  c: Integer;
  index, color: Integer;
begin
  r := TRegExpr.Create;
  fillchar(colors, sizeof(colors), -1);
  c := 0;
  try
    r.InputString := text;
    r.Expression := 'SetPlayerColor\s*\(\s*Player\s*\(\s*(\d*)\s*\)\s*,\s*ConvertPlayerColor\s*\(\s*(\d*)\s*\)\s*\)';
    if r.Exec then
    begin
      inc(c);
      index := StrToInt(r.Match[1]);
      color := StrToInt(r.Match[2]);
      colors[index] := color;
      while r.ExecNext do begin
        inc(c);
        index := StrToInt(r.Match[1]);
        color := StrToInt(r.Match[2]);
        colors[index] := color;
      end;
    end;
  finally
    r.Free;
  end;
  result := c > 0;
end;

function TMapInfo.GetTeams(text: PChar): boolean;
var
  r: TRegExpr;
  c: Integer;
  index, team: Integer;
begin
  fillchar(teams, sizeof(teams), -1);
  c := 0;
  r := TRegExpr.Create;
  try
    r.InputString := text;
    r.Expression := 'SetPlayerTeam\s*\(\s*Player\s*\(\s*(\d*)\s*\)\s*,\s*(\d*)\s*\)';
    if r.Exec then
    begin
      inc(c);
      index := StrToInt(r.Match[1]);
      team := StrToInt(r.Match[2]);
      teams[index] := team;
      while r.ExecNext do begin
        inc(c);
        index := StrToInt(r.Match[1]);
        team := StrToInt(r.Match[2]);
        teams[index] := team;
      end;
    end;
  finally
    r.Free;
  end;
  result := c > 0;
end;

function TMapInfo.ReadMap(FileNameOfMap: String): boolean;
var
  mpqhandle, filehandle: THandle;
  count, i: Integer;
  filelist: array of FILELISTENTRY;
  size, read: Cardinal;
  P: PChar;
  filename: String;
  start: Cardinal;
begin
  result := true;
  start := GetTickCount;
  Clear;
  try
    if SFileOpenArchive(PChar(FileNameOfMap), 0, 0, @mpqhandle) then
    try
      count := SFileGetFileInfo(mpqhandle, SFILE_INFO_HASH_TABLE_SIZE);
      SetLength(filelist, count);
      if SFileListFiles(mpqhandle, nil, @filelist[0], 0) then begin
        for i := 0 to count - 1 do
        if filelist[i].dwFileExists <> 0 then begin
          size := filelist[i].dwFullSize;
          filename := filelist[i].szFileName;
          getmem(p, size + 1);
          try
            fillchar(p^, size + 1, 0);
            if SFileOpenFileEx(mpqhandle, pchar(filename), 0, @filehandle) then
            try
              if SFileReadFile(filehandle, p, size, @read, nil) then
              begin
                if pos('function config takes nothing returns nothing', p)>0 then
                begin
                  playerCount := CountPlayers(p);
                  colorsOK := GetColors(p);
                  teamsOK := GetTeams(p);
                end;
              end;
            finally
              SFileCloseFile(filehandle);
            end;
          finally
            freemem(p);
          end;
          if (playerCount > 0) and colorsOK and teamsOK then break;
          if GetTickCount - start > 3000 then
          begin
            result := false;
            break;
          end;
        end;
      end;
    finally
      SFileCloseArchive(mpqhandle);
    end;
  except
    result := false;
  end;
end;

end.
 