unit UPacket;

interface
uses windows, sysutils, classes;

type
  TCPData = array[0..16024] of byte;

  Packet = record
    a, b: longword;
    pa, pb: word;
    l: word;
    d: ^TCPData;
  end;

  PPacket = ^Packet;

  TPacket = class(TObject)
  public
    p: PPacket;
    constructor Create;
    destructor Destroy; override;
    function LoadFrom(src: Packet): boolean;
    function GetByte(i: word): byte;
    function GetCardinal(i: word): Cardinal;
    function GetString(i: word): string;
    function Length: Word;
    function Src: string;
    function SrcIP: Cardinal;
    function SrcPort: Word;
    function Dest: string;
    function DestIP: Cardinal;
    function DestPort: Word;
    function Where: Pointer;
  end;

  TPacketQueue = class(TObject)
  private
    packets: TThreadList;
  public
    function  packetCount: integer;
    procedure enqueuePacket(p: PPacket);
    function  dequeuePacket(var p: TPacket): boolean;
    procedure clearPacketQueue;
    constructor Create;
    destructor Destroy; override;
  end;


var
  packets: TThreadList;

  // packet constructor and destructor
procedure createPacket(size: Word; clear: boolean; var dest: PPacket);
procedure freePacket(p: Packet);

function IPtoStr(IP: pbyte): string;
procedure BytesToIP(b1, b2, b3, b4: byte; ip: Pointer);

// thread secure packet queue
function  packetCount: integer;
procedure enqueuePacket(p: PPacket);
function  dequeuePacket(var p: TPacket): boolean;
procedure clearPacketQueue;

implementation

function IPtoStr(IP: pbyte): string;
type
  PIP = ^TIP;
  TIP = array[0..3] of byte;
begin
  result := format('%d.%d.%d.%d', [PIP(IP)[0],
    PIP(IP)[1],
      PIP(IP)[2],
      PIP(IP)[3]]);
end;

procedure BytesToIP(b1, b2, b3, b4: byte; ip: Pointer);
type
  PIP = ^TIP;
  TIP = array[0..3] of byte;
begin
  PIP(IP)[0] := b1;
  PIP(IP)[1] := b2;
  PIP(IP)[2] := b3;
  PIP(IP)[3] := b4;
end;

procedure createPacket(size: Word; clear: boolean; var dest: PPacket);
begin
  New(dest);
  GetMem(dest^.d, Size);
  if clear then
  begin
    dest^.a := 0;
    dest^.b := 0;
    dest^.pa := 0;
    dest^.pb := 0;
    ZeroMemory(dest^.d, size);
  end;
  dest^.l := size;
end;

procedure freePacket(p: Packet);
begin
  if p.d <> nil then
    FreeMem(p.d);
  p.d := nil;
end;

{ TPacket }

constructor TPacket.Create;
begin
  p := nil;
end;

function TPacket.Dest: string;
begin
  result := iptostr(@p^.b);
end;

function TPacket.GetByte(i: word): byte;
begin
  if i >= p^.l then
    raise Exception.Create('Index out of bounds while reading byte from packet.')
  else
    result := p^.d^[i];
end;

function TPacket.GetString(i: word): string;
begin
  result := '';
  while i < p^.l do
    if p^.d^[i] = 0 then
      break
    else
    begin
      result := result + chr(p^.d^[i]);
      inc(i);
    end;
  if not (i < p^.l) then
    raise
      Exception.Create('String out of bounds while reading string from packet.');
end;

function TPacket.Length: Word;
begin
  result := p^.l;
end;

function TPacket.LoadFrom(src: Packet): boolean;
begin
  if p <> nil then
    Free;
  if src.d = nil then
    result := false
  else
  begin
    CreatePacket(src.l, false, p);
    p^.a := src.a;
    p^.b := src.b;
    CopyMemory(p^.d, src.d, src.l);
    p^.l := src.l;
    result := true;
  end;
end;

function TPacket.Where: Pointer;
begin
  result := p^.d;
end;

function TPacket.Src: string;
begin
  result := iptostr(@p^.a);
end;

function packetCount: integer;
begin
  result := packets.locklist.Count;
  packets.UnlockList;
end;

procedure enqueuePacket(p: PPacket);
begin
  packets.Add(p);
end;

function dequeuePacket(var p: TPacket): boolean;
var
  li: TList;
  pp: PPacket;
begin
  if packetCount = 0 then
    result := false
  else
  begin
    p := TPacket.Create;
    li := Packets.LockList;
    pp := li.first;
    p.p := pp;
    li.Delete(0);
    packets.UnlockList;
    result := true;
  end;
end;

procedure clearPacketQueue;
var
  p: TPacket;
begin
  while dequeuePacket(p) do
  begin
    p.Free;
  end;
end;

function TPacket.DestIP: Cardinal;
begin
  result := p^.b;
end;

function TPacket.SrcIP: Cardinal;
begin
  result := p^.a;
end;

function TPacket.DestPort: Word;
begin
  result := p^.pb;
end;

function TPacket.SrcPort: Word;
begin
  result := p^.pa;
end;

destructor TPacket.Destroy;
begin
  FreePacket(p^);
  Dispose(p);
  p := nil;
  inherited;
end;

function TPacket.GetCardinal(i: word): Cardinal;
begin
  if i + 3 >= p^.l then
    raise Exception.Create('Index out of bounds while reading cardinal from packet.')
  else
    result := PCardinal(@p^.d^[i])^;
end;

{ TPacketQueue }

function TPacketQueue.packetCount: integer;
begin
  result := packets.locklist.Count;
  packets.UnlockList;
end;

procedure TPacketQueue.enqueuePacket(p: PPacket);
begin
  packets.Add(p);
end;

function TPacketQueue.dequeuePacket(var p: TPacket): boolean;
var
  li: TList;
  pp: PPacket;
begin
  if packetCount = 0 then
    result := false
  else
  begin
    p := TPacket.Create;
    li := Packets.LockList;
    pp := li.first;
    p.p := pp;
    li.Delete(0);
    packets.UnlockList;
    result := true;
  end;
end;

procedure TPacketQueue.clearPacketQueue;
var
  p: TPacket;
begin
  while dequeuePacket(p) do
  begin
    p.Free;
  end;
end;

constructor TPacketQueue.Create;
begin
  packets := TThreadList.Create;
end;

destructor TPacketQueue.Destroy;
begin
  clearPacketQueue;
  packets.Free;
  inherited;
end;

initialization

  packets := TThreadList.Create;

finalization

  clearPacketQueue;
  packets.free;

end.
