unit UPacketReceiver;

interface

uses
  zniffer, upacket, sysutils, windows, winsock, wc3types;

  type
    TPacketReceiver = class(TObject)
    private
      procedure ReadPacket(Data: pointer; recvbytes: Word; uTickCount: Int64);
    public
      sniffer: TZniffer;
      myip: IPArray;
      ipcount: word;
      pq: TPacketQueue;
      trafficPPPoE, trafficTCPin, trafficTCPout, trafficWC3in,
        trafficWC3out,  trafficAny, trafficUDPin, trafficUDPout: Integer;
      constructor Create(index: Integer);
      destructor Destroy; override;
      procedure Sniff;
      function InMyIp(ip: dword): boolean; overload;
      function InMyIp(ip: string): boolean; overload;
      procedure getIPs(var a: iparray; var c: word);
    end;

implementation

{ TPacketReceiver }

procedure TPacketReceiver.getIPs(var a: iparray; var c: word);
var
  i, j, p: Integer;
  s: String;
begin
  c := 0;
  for i := 0 to sniffer.Addresses.Count - 1 do
  begin
    s := sniffer.Addresses[i];
    if s <> '' then
    begin
      a[c] := 0;
      j := 0;
      repeat
        p := pos('.', s);
        if p = 0 then
        begin
          a[c] := a[c] or (StrToInt(s) and $FF) shl j;
          inc(j, 8);
        end else
        begin
          a[c] := a[c] or (StrToInt(Copy(s, 1, p - 1)) and $FF) shl j;
          inc(j, 8);
          delete(s, 1, p);
        end;
      until j = 32;
      inc(c);
    end;
  end;
end;

function TPacketReceiver.InMyIp(ip: string): boolean;
var
  i: integer;
  addr: dword;
begin
  result := false;
  addr := inet_addr(pchar(ip));
  for i := 0 to ipcount - 1 do
    if (addr = myip[i]) then
    begin
      result := true;
      break;
    end;
end;

function TPacketReceiver.InMyIp(ip: dword): boolean;
var
  i: integer;
begin
  result := false;
  for i := 0 to ipcount - 1 do
    if (ip = myip[i]) then
    begin
      result := true;
      break;
    end;
end;


constructor TPacketReceiver.Create(index: Integer);
begin
  pq := TPacketQueue.Create;
  sniffer := TZniffer.Create;
  sniffer.AdapterIndex := index;
  sniffer.SnapLen := Word(-1);
  sniffer.Promisc := true;
  sniffer.OnPacket := ReadPacket;
  getIPs(myip, ipcount);
end;

destructor TPacketReceiver.Destroy;
var
  e: String;
begin
  if sniffer.Snooping then
    sniffer.Deactivate(e);
  sniffer.Free;
  pq.Free;
  inherited;
end;

procedure TPacketReceiver.ReadPacket(Data: pointer; recvbytes: Word; uTickCount: Int64);
type
  ba = array[0..100] of byte;
  pba = ^ba;
var
  p: PPacket;
  IP: PIP_RHDR;
  rp, lp: DWORD;
  tos: byte;
  raw: PByteArray;
  invalid: boolean;
  i, size: Integer;
  t: TPacket;
  ipOffset, tcpOffset, tcpDataOffset, tcpSize: WORD;
  dirIn, dirOut: boolean;

  procedure incdword(p: pba; i: word);
  var
    u: word;
  begin
    u := (p^[3] + i) shr 8;
    p^[3] := (p^[3] + i) and $FF;
    i := u;
    u := (p^[2] + i) shr 8;
    p^[2] := (p^[2] + i) and $FF;
    i := u;
    u := (p^[1] + i) shr 8;
    p^[1] := (p^[1] + i) and $FF;
    i := u;
    p^[0] := (p^[0] + i) and $FF;
  end;

begin
  inc(trafficAny);
  // Initialize
  raw := Data;
  dirIn := false;
  dirOut := false;
  // find out the layer 3+4 protocol offsets
  if (raw^[12] = $08) and (raw^[13] = $00) then
  begin
    ipOffset := 14; // <--- size of a Ethernet II Header
  end
  else if (raw^[12] = $88) and (raw^[13] = $64)
    and (raw^[$0F] = 0) and (raw^[$14] = 0) and (raw^[$15] = $21) then
  begin
    ipOffset := 22;
    inc(trafficPPPoE);
  end
  else
    exit; // unknown TOS field in Ethernet II Frame
  IP := @raw^[ipOffset];
  if inmyip(iptostr(@ip.srcIP)) then
    dirOut := true;
  if inmyip(iptostr(@ip.destIP)) then
    dirIn := true;
  tos := raw^[IpOffset + 9];
  if tos = $11 then
  begin
    if dirOut then inc(trafficUDPout);
    if dirIn then inc(trafficUDPin);
  end;
  if tos <> 6 then
    exit; // TCP Packet?
  if dirOut then inc(trafficTCPout);
  if dirIn then inc(trafficTCPin);
  // calc the other offsets
  tcpOffset := ipOffset + ((raw^[ipOffset] and $0F) shl 2); // + IP Header Size
  tcpDataOffset := tcpOffset + ((raw^[tcpOffset + 12] and $F0) shr 2);
  // + TCP Header Size
  tcpSize := raw^[ipOffset + 2] shl 8 + raw^[ipOffset + 3] + ipOffset -
    tcpDataOffset;
  // create packet
  CreatePacket(tcpSize, false, p);
  // copy only TCP data
  CopyMemory(p^.d, @raw^[tcpDataOffset], tcpSize);
  // copy IP addresses
  CopyMemory(@p^.a, @ip.srcIP, 4);
  CopyMemory(@p^.b, @ip.destIP, 4);
  // copy ports
  rp := raw^[tcpOffset] shl 8 + raw^[tcpOffset + 1];
  p^.pa := rp;
  lp := raw^[tcpOffset + 2] shl 8 + raw^[tcpOffset + 3];
  p^.pb := lp;
  // enqueue the packet
  p^.l := tcpSize;
  pq.enqueuePacket(p);
  pq.dequeuePacket(t);
  invalid := true;
  try
    i := 0;
    // Packet Loop
    if (t.Length > i) and (t.GetByte(i) in [$F7, $FF]) then
      while i + 3 < t.Length do
      begin // +3 because of size-bytes following the command code
        invalid := false;
        size := t.GetByte(i + 2) + t.GetByte(i + 3) shl 8;
        // check for invalid packets
        if (size = 0) or (i + size > t.Length) or ((i + size < t.Length) and
          not (t.GetByte(i + size) in [$F7, $FF, $0])) then
          invalid := true;
        inc(i, size);
      end;
  except
  end;
  t.free;
  if not invalid then
  begin
    if dirIn then inc(trafficWC3in);
    if dirOut then inc(trafficWC3out);
  end;
end;

procedure TPacketReceiver.Sniff;
var
  e: String;
begin
  sniffer.Activate(e);
  if sniffer.Snooping = false then
  raise exception.Create(e);
end;

end.
