unit UReserve;

interface

uses classes, inifile, WC3Types, SysUtils;

type
  TReserve = class(TObject)
  private
    FReservations: TStringList;
  public
    constructor Create;
    destructor Free;
    function CanJoin(nick: string; freeSlots: Integer): boolean;
    procedure Clear;
    function Reserve(pattern: string): boolean;
    function Reservations: String;
    function List: TStringlist;
  end;

implementation

{ TReserve }

function TReserve.CanJoin(nick: string; freeSlots: Integer): boolean;
var i: Integer;
    patternFound: Boolean;
    patternMatch: Boolean;
begin
  if freeSlots = 0 then
    result := true
  else begin
    patternMatch := false;
    patternFound := false;
    for i := 0 to FReservations.Count - 1 do
    begin
      if pos('*', FReservations[i]) > 0 then
        patternFound := true;
      if MatchesPattern(nick, FReservations[i]) then begin
        patternMatch := true;
        // Remove one-player-reservations
        if pos('*', FReservations[i]) = 0 then FReservations.Delete(i);
        break;
      end;
    end;
    if patternMatch then result := true
    else if patternFound then result := false
    else if (freeSlots > FReservations.Count) then result := true
    else result := false;
  end;
end;

procedure TReserve.Clear;
begin
  FReservations.Clear;
end;

constructor TReserve.Create;
begin
  FReservations := TStringlist.Create;
end;

destructor TReserve.Free;
begin
  FReservations.Free;
end;

function TReserve.List: TStringlist;
begin
  result := FReservations;
end;

function TReserve.Reservations: String;
var i, c: Integer;
begin
  c := FReservations.Count;
  if c = 0 then
    result := 'No reservations recorded.'
  else begin
    result := 'Slots are reserved for: ';
    for i := 0 to c - 1 do
    begin
      result := result + FReservations[i];
      if i < c - 1 then result := result + ', ';
    end;
  end;
end;

function TReserve.Reserve(pattern: string): boolean;
var
  p: String;
  i: Integer;
begin
  FReservations.CaseSensitive := false;
  if FReservations.IndexOf(pattern) = -1 then
  begin
    for i := 1 to length(pattern) do
      if pattern[i] <> '*' then
        p := p + pattern[i];
    result := CheckNicknameSpelling(p);
    if result then
      FReservations.Add(lowercase(pattern));
  end else
    result := true;
end;

end.
