unit URestore;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls,
  Forms, Dialogs, ExtCtrls, VirtualTrees, ComCtrls, IdHTTP,
  IdCompressorZLibEX, ProxyInfo, IniFile, StdCtrls, ShellAPI, UListContainer;

const
  typStr: Array[0..3] of String =
    ('Ban', 'Deleted Ban', 'Info', 'Deleted Info');

type
  TEntry = record
    nick: String;
    comment: String;
    typ: Integer;
  end;

  PEntry = ^TEntry;

  TfrmRestore = class(TForm)
    StatusBar: TStatusBar;
    vstDB: TVirtualStringTree;
    pnTop: TPanel;
    tmDownload: TTimer;
    btnMarkLost: TButton;
    btnCopySelection: TButton;
    Label1: TLabel;
    lblLink: TLabel;
    procedure tmDownloadTimer(Sender: TObject);
    procedure vstDBGetNodeDataSize(Sender: TBaseVirtualTree;
      var NodeDataSize: Integer);
    procedure vstDBFreeNode(Sender: TBaseVirtualTree;
      Node: PVirtualNode);
    procedure vstDBGetText(Sender: TBaseVirtualTree; Node: PVirtualNode;
      Column: TColumnIndex; TextType: TVSTTextType;
      var CellText: WideString);
    procedure vstDBGetImageIndex(Sender: TBaseVirtualTree;
      Node: PVirtualNode; Kind: TVTImageKind; Column: TColumnIndex;
      var Ghosted: Boolean; var ImageIndex: Integer);
    procedure vstDBHeaderClick(Sender: TVTHeader; Column: TColumnIndex;
      Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure vstDBCompareNodes(Sender: TBaseVirtualTree; Node1,
      Node2: PVirtualNode; Column: TColumnIndex; var Result: Integer);
    procedure btnMarkLostClick(Sender: TObject);
    procedure btnCopySelectionClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure lblLinkMouseEnter(Sender: TObject);
    procedure lblLinkMouseLeave(Sender: TObject);
    procedure lblLinkClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormShow(Sender: TObject);
    procedure FormResize(Sender: TObject);
  private
    { Private-Deklarationen }
  public
    procedure DownloadDatabase;
    procedure Status(s: String);
    procedure AddEntry(nick: String; comment: String; typ: Integer);
    { Public-Deklarationen }
  end;

var
  frmRestore: TfrmRestore;

implementation

uses Unit1;

{$R *.dfm}

procedure TfrmRestore.DownloadDatabase;
var
  t, s: tstringlist;
  login, pass, sessionid: string;
  i: integer;
  idhttp1: tidhttp;
  proxy: string;
  port: integer;
  IdCompressorZLibEX1: TIdCompressorZLibEX;
begin
  vstDB.Clear;
  Status('Preparing connection to www.banlist.nl ...');
  idhttp1 := tidhttp.Create(self);
  idhttp1.Request.AcceptEncoding := 'gzip, deflate, ';
  IdCompressorZLibEX1 := TIdCompressorZLibEX.Create;
  idhttp1.Compressor := IdCompressorZLibEX1;
  if WC3Banlist_Main.chkProxy.Checked then
  begin
    getproxyserver('http', proxy, port);
    idhttp1.ProxyParams.ProxyServer := proxy;
    try
      idhttp1.ProxyParams.ProxyPort := port;
    except
      Status('This is no valid proxy port!');
      exit;
    end;
  end
  else
  begin
    idhttp1.ProxyParams.ProxyServer := '';
    idhttp1.ProxyParams.ProxyPort := 0;
  end;

  login := WC3Banlist_Main.edtBlLogin.Text;
  pass := WC3Banlist_Main.edtBlPassword.Text;
  Status('Logging in to Buddies'' Banlists ...');

  sessionid := idhttp1.Get(WC3Banlist_Main.edtURLLogin.Text +
    '?login=' + login + '&pass=' + pass);
  if pos(' ', sessionid) <> 0 then
  begin
    Status('Login to Buddies'' Banlists failed. Response:' +
      #13#10 + sessionid);
  end
  else
  begin
    // download buddies' lists
    t := tstringlist.create;
    s := tstringlist.create;
    try
      vstDB.BeginUpdate;
      Status('Downloading your local database ...');
      t.Text := idhttp1.Get(WC3Banlist_Main.edtURLDownload.Text +
        '?list=own&sid=' + sessionid);
      for i := 0 to t.Count - 1 do
      begin
        Split(t[i], #9, s);
        try
          AddEntry(s[0], s[1], StrToInt(s[3]));
        except
          raise Exception.Create('Format failure in line ' + inttostr(i) + #13#10
            + '[' + t[i] + ']');
        end;
      end;
      Status('Done. ' + IntToStr(vstDB.TotalCount) + ' entries found.');
    finally
      t.free;
      s.free;
      idhttp1.Free;
      IdCompressorZLibEX1.Free;
      vstDB.EndUpdate;
    end;
  end;
end;

procedure TfrmRestore.Status(s: String);
begin
  StatusBar.SimpleText := s;
end;

procedure TfrmRestore.tmDownloadTimer(Sender: TObject);
begin
  tmDownload.Enabled := false;
  DownloadDatabase;
end;

procedure TfrmRestore.vstDBGetNodeDataSize(Sender: TBaseVirtualTree;
  var NodeDataSize: Integer);
begin
  NodeDataSize := sizeof(TEntry);
end;

procedure TfrmRestore.vstDBFreeNode(Sender: TBaseVirtualTree;
  Node: PVirtualNode);
var
  p: PEntry;
begin
  p := Sender.GetNodeData(Node);
  p^.nick := '';
  p^.comment := '';
end;

procedure TfrmRestore.AddEntry(nick, comment: String; typ: Integer);
var
  p: PEntry;
  n: PVirtualNode;
begin
  n := vstDB.AddChild(nil);
  p := vstDB.GetNodeData(n);
  vstDB.ValidateNode(n, false);
  p^.nick := nick;
  p^.comment := comment;
  p^.typ := typ;
end;

procedure TfrmRestore.vstDBGetText(Sender: TBaseVirtualTree;
  Node: PVirtualNode; Column: TColumnIndex; TextType: TVSTTextType;
  var CellText: WideString);
var
  p: PEntry;
begin
  p := vstDB.GetNodeData(Node);
  case Column of
    0: CellText := p^.nick;
    1: CellText := p^.comment;
    2: CellText := typStr[p^.typ];
  end;
end;

procedure TfrmRestore.vstDBGetImageIndex(Sender: TBaseVirtualTree;
  Node: PVirtualNode; Kind: TVTImageKind; Column: TColumnIndex;
  var Ghosted: Boolean; var ImageIndex: Integer);
var
  p: PEntry;
begin
  if Column = 0 then
  begin
    p := vstDB.GetNodeData(Node);
    ImageIndex := p^.typ;
  end;
end;

procedure TfrmRestore.vstDBHeaderClick(Sender: TVTHeader;
  Column: TColumnIndex; Button: TMouseButton; Shift: TShiftState; X,
  Y: Integer);
begin
  if Button = mbLeft then
  begin
    with Sender do
    begin
      if SortColumn <> Column then
      begin
        SortColumn := Column;
        SortDirection := sdAscending;
      end
      else if SortDirection = sdAscending then
        SortDirection := sdDescending
      else
        SortDirection := sdAscending;
    end;
  end;
end;

procedure TfrmRestore.vstDBCompareNodes(Sender: TBaseVirtualTree; Node1,
  Node2: PVirtualNode; Column: TColumnIndex; var Result: Integer);
var
  p1, p2: PEntry;
begin
  result := 0;
  p1 := Sender.GetNodeData(Node1);
  p2 := Sender.GetNodeData(Node2);
  case column of
    0: result := compareStr(lowercase(p1^.nick), lowercase(p2^.nick));
    1: result := compareStr(lowercase(p1^.comment), lowercase(p2^.comment));
    2: result := p1^.typ - p2^.typ;
  end;
end;

procedure TfrmRestore.btnMarkLostClick(Sender: TObject);
var
  c: Integer;
  n: PVirtualNode;
  p: PEntry;
  ban: PBan;
  info: PInfo;
begin
  c := 0;
  vstDB.ClearSelection;
  n := vstDB.GetFirst;
  while Assigned(n) do
  begin
    p := vstDB.GetNodeData(n);
    ban := WC3Banlist_Main.ExistsOnLocalBanlist(p^.nick);
    info := WC3Banlist_Main.ExistsOnLocalInfolist(p^.nick);
    if ((p^.typ in [0,1]) and not Assigned(ban))
      or ((p^.typ in [2,3]) and not Assigned(info))
    then
    begin
      inc(c);
      vstDB.Selected[n] := true;
    end;
    n := vstDB.GetNext(n);
  end;
  if c=0 then
    Status('No lost entries found. Your local database is complete.')
  else if c=1 then
    Status(IntToStr(c)+' lost entry found.')
  else
    Status(IntToStr(c)+' lost entries found.');
  if c > 0 then
  begin
    n := vstDB.GetFirstSelected;
    vstDB.ScrollIntoView(n, true);
  end;
end;

procedure TfrmRestore.btnCopySelectionClick(Sender: TObject);
var
  c: Integer;
  n: PVirtualNode;
  p: PEntry;
  ban: PBan;
  info: PInfo;
begin
  c := 0;
  n := vstDB.GetFirstSelected;
  while Assigned(n) do
  begin
    p := vstDB.GetNodeData(n);
    if p^.typ in [0,1] then begin
      new(ban);
      ban^.nick := p^.nick;
      ban^.comment := p^.comment;
      ban^.deleted := Odd(p^.typ);
      ban^.lastChange := now;
      ListContainer.banlist.AddEntry(ban);
    end else begin
      new(info);
      info^.nick := p^.nick;
      info^.comment := p^.comment;
      info^.deleted := Odd(p^.typ);
      info^.lastChange := now;
      ListContainer.infolist.AddEntry(info);
    end;
    inc(c);
    n := vstDB.GetNextSelected(n);
  end;
  if (c=0) or (c>1) then
    Status(IntToStr(c) + ' entries have been restored.')
  else
    Status('1 entry has been restored.');
  WC3Banlist_Main.RefreshLocalDatabase;
end;

procedure TfrmRestore.FormCreate(Sender: TObject);
begin
  SetFormFont(self, defaultFont, defaultFontSize);
  vstDB.Header.Font.Name := defaultFont;
  SetSunkenButtons(self);
end;

procedure TfrmRestore.lblLinkMouseEnter(Sender: TObject);
begin
  (Sender as TLabel).Font.Style :=
    (Sender as TLabel).Font.Style + [fsUnderline];
end;

procedure TfrmRestore.lblLinkMouseLeave(Sender: TObject);
begin
  (Sender as TLabel).Font.Style :=
    (Sender as TLabel).Font.Style - [fsUnderline];
end;

procedure TfrmRestore.lblLinkClick(Sender: TObject);
begin
  ShellExecute(0, 'open', PChar(TLabel(Sender).Hint), nil, nil, sw_show);
end;

procedure TfrmRestore.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  icd.SaveCoords(self);
end;

procedure TfrmRestore.FormShow(Sender: TObject);
begin
  icd.LoadCoords(self);
end;

procedure TfrmRestore.FormResize(Sender: TObject);
begin
  Label1.Left := pnTop.Width - Label1.Width - lblLink.Width - 10;
  lblLink.Left := pnTop.Width - lblLink.Width - 7;
end;

end.
