unit UfrmGameReview;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, VirtualTrees, Buttons, IniFile, UListContainer, WC3Types,
  ShellAPI, ExtCtrls;

type
  PEntry = ^TEntry;
  TEntry = record
    nick: String;
    score: String;
  end;

  TfrmGameReview = class(TForm)
    Panel: TPanel;
    vtScores: TVirtualStringTree;
    gbGameInformation: TGroupBox;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    Label5: TLabel;
    Label6: TLabel;
    lblGameTitle: TLabel;
    lblMapName: TLabel;
    lblOwnNickname: TLabel;
    lblRealm: TLabel;
    lblStartTime: TLabel;
    lblDuration: TLabel;
    gbScreenshots: TGroupBox;
    btnViewScreenshot: TSpeedButton;
    btnDeleteScreenshot: TSpeedButton;
    btnOpenScreenshotFolder: TSpeedButton;
    lbScreenshots: TListBox;
    gbReplay: TGroupBox;
    lblReplay: TLabel;
    btnViewReplay: TSpeedButton;
    btnRate1: TButton;
    btnRate2: TButton;
    btnRate3: TButton;
    btnRate4: TButton;
    btnRate5: TButton;
    btnRate6: TButton;
    btnSubmit: TButton;
    btnClose: TButton;
    gcComment: TGroupBox;
    memoComment: TMemo;
    Label7: TLabel;
    lblHost: TLabel;
    procedure FormCreate(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure btnCloseClick(Sender: TObject);
    procedure vtScoresBeforeCellPaint(Sender: TBaseVirtualTree;
      TargetCanvas: TCanvas; Node: PVirtualNode; Column: TColumnIndex;
      CellRect: TRect);
    procedure vtScoresGetText(Sender: TBaseVirtualTree; Node: PVirtualNode;
      Column: TColumnIndex; TextType: TVSTTextType;
      var CellText: WideString);
    procedure vtScoresFocusChanged(Sender: TBaseVirtualTree;
      Node: PVirtualNode; Column: TColumnIndex);
    procedure btnRateClick(Sender: TObject);
    procedure btnViewReplayClick(Sender: TObject);
    procedure btnViewScreenshotClick(Sender: TObject);
    procedure btnDeleteScreenshotClick(Sender: TObject);
    procedure btnOpenScreenshotFolderClick(Sender: TObject);
    procedure memoCommentChange(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure vtScoresPaintText(Sender: TBaseVirtualTree;
      const TargetCanvas: TCanvas; Node: PVirtualNode;
      Column: TColumnIndex; TextType: TVSTTextType);
    procedure btnSubmitClick(Sender: TObject);
    procedure vtScoresClick(Sender: TObject);
  private
    procedure SetButtonsEnabled;
  public
    game: PGame;
    procedure RefreshReview;
  end;

var
  frmGameReview: TfrmGameReview;

implementation

uses unit1;

{$R *.dfm}

procedure TfrmGameReview.FormCreate(Sender: TObject);
begin
  SetFormFont(self, defaultFont, defaultFontSize);
  vtScores.NodeDataSize := SizeOf(TEntry);
end;

procedure TfrmGameReview.RefreshReview;
var
  nicks, scores, screenshots: TStringlist;
  i: Integer;
  v: Integer;
  n: PVirtualNode;
  p: PEntry;
begin
  with vtScores do
  begin
    BeginUpdate;
    Clear;
    nicks := TStringlist.Create;
    scores := TStringlist.Create;
    try
      split(game^.players, '|', nicks);
      split(game^.givenPoints, '|', scores);
      for i := 0 to nicks.count - 1 do
      begin
        n := AddChild(nil);
        p := GetNodeData(n);
        ValidateNode(n, false);
        p^.nick := nicks[i];
        if scores[i] = '0' then
          p^.score := ''
        else begin
          v := StrToInt(scores[i]);
          if v < 0 then p^.score := '- ' + IntToStr(v * -1);
          if v > 0 then p^.score := '+ ' + IntToStr(v);
        end;
      end;
    finally
      EndUpdate;
      nicks.free;
      scores.free;
    end;
  end;
  lblGameTitle.Caption := game^.gameName;
  lblMapName.Caption := game^.mapName;
  lblOwnNickname.Caption := game^.ownNickName;
  lblRealm.Caption := game^.realm;
  lblHost.Caption := game^.host;
  lblStartTime.Caption := datetimetostr(game^.gameStart);
  lblDuration.Caption := game^.duration;
  if game^.replay <> '' then
    lblReplay.Caption := game^.replay;
  btnViewReplay.Enabled := game^.replay <> '';
  screenshots := TStringlist.create;
  try
    split(game^.screenShots, '|', screenshots);
    lbScreenshots.Items.Assign(screenshots);
  finally
    screenshots.Free;
  end;
  memoComment.OnChange := nil;
  memoComment.text := game^.comment;
  memoComment.OnChange := memoCommentChange;
end;

procedure TfrmGameReview.FormShow(Sender: TObject);
begin
  icd.LoadCoords(self);
  RefreshReview;
end;

procedure TfrmGameReview.btnCloseClick(Sender: TObject);
begin
  close;
end;

procedure TfrmGameReview.vtScoresBeforeCellPaint(Sender: TBaseVirtualTree;
  TargetCanvas: TCanvas; Node: PVirtualNode; Column: TColumnIndex;
  CellRect: TRect);
var
  i: Integer;
  p: PEntry;
begin
  i := node.Index;
  p := Sender.GetNodeData(node);
  if (column = 0) or (p^.Nick = 'Computer') then
  begin
    TargetCanvas.Brush.Color := lvGameColors[i];
    targetcanvas.FillRect(cellrect);
  end;
end;

procedure TfrmGameReview.vtScoresGetText(Sender: TBaseVirtualTree;
  Node: PVirtualNode; Column: TColumnIndex; TextType: TVSTTextType;
  var CellText: WideString);
var
  i: Integer;
  p: PEntry;
begin
  i := node.Index;
  p := Sender.GetNodeData(node);
  case column of
    0: CellText := IntToStr(i + 1);
    1: CellText := p^.nick;
    2: CellText := p^.score;
  end;
end;

procedure TfrmGameReview.SetButtonsEnabled;
var
  b: Boolean;
  n: PVirtualNode;
  p: PEntry;
begin
  n := vtScores.GetFirstSelected;
  b := assigned(n);
  while assigned(n) do
  begin
    p := vtScores.GetNodeData(n);
    b := b and (p^.nick <> 'Computer') and (p^.nick <> 'Open') and (p^.nick <> 'Closed') and (p^.nick <> game^.ownNickName);
    n := vtScores.GetNextSelected(n);  
  end;
  btnRate1.Enabled := b;
  btnRate2.Enabled := b;
  btnRate3.Enabled := b;
  btnRate4.Enabled := b;
  btnRate5.Enabled := b;
  btnRate6.Enabled := b;
end;

procedure TfrmGameReview.vtScoresFocusChanged(Sender: TBaseVirtualTree;
  Node: PVirtualNode; Column: TColumnIndex);
begin
  SetButtonsEnabled;
end;

procedure TfrmGameReview.btnRateClick(Sender: TObject);
var
  p: PEntry;
  i: Integer;
  n: PVirtualNode;
  scores: TStringlist;
begin
  n := vtScores.GetFirstSelected;
  while assigned(n) do
  begin
    p := vtScores.GetNodeData(n);
    if listcontainer.scorelist.FindEntry(p^.nick, i) then
    begin
      scores := TStringlist.Create;
      try
        split(game^.givenPoints, '|', scores);
        WC3Banlist_Main.AddRating(p^.nick, TButton(Sender).Tag, StrToInt(Scores[n.Index]), game^.id);
        p^.score := RatingToStr(TButton(Sender).Tag);
        vtScores.InvalidateNode(n);
      finally
        scores.Free;
      end;
    end else
      MessageDlg('The selected player was not found in the scores database.'+#13+#10+''+#13+#10+'You can only rate players you have played with.', mtInformation, [mbOK], 0);
    n := vtScores.GetNextSelected(n);
  end;
end;

procedure TfrmGameReview.btnViewReplayClick(Sender: TObject);
begin
  ShellExecute(0, 'open',
    Pchar(IniPreparePath(WC3Banlist_Main.edtReplayDatabase.Text, '\') +
      lblReplay.Caption), nil, nil, SW_SHOW);
end;

procedure TfrmGameReview.btnViewScreenshotClick(Sender: TObject);
var
  path: String;
  filename: String;
begin
  if lbScreenshots.ItemIndex = -1 then exit;
  path := IniPreparePath(WC3Banlist_Main.edScreenshotDBLocation.Text, '\');
  filename := lbScreenshots.Items[lbScreenshots.itemIndex];
  ShellExecute(0, 'open', Pchar(Path + FileName), nil, nil, SW_SHOW);
end;

procedure TfrmGameReview.btnDeleteScreenshotClick(Sender: TObject);
var
  path: String;
  filename: String;
  s: TStringList;
begin
  if lbScreenshots.ItemIndex = -1 then exit;
  path := IniPreparePath(WC3Banlist_Main.edScreenshotDBLocation.Text, '\');
  filename := lbScreenshots.Items[lbScreenshots.itemIndex];
  s := TStringlist.Create;
  try
    DeleteFile(Path + Filename);
    split(game^.screenShots, '|', s);
    s.Delete(lbScreenshots.ItemIndex);
    lbScreenshots.Items.Assign(s);
    game^.screenShots := join(s, '|');
  finally
    s.Free;
  end;
end;

procedure TfrmGameReview.btnOpenScreenshotFolderClick(Sender: TObject);
var
  path: String;
begin
  ForceDirectories(WC3Banlist_Main.edScreenshotDBLocation.Text);
  path := IniPreparePath(WC3Banlist_Main.edScreenshotDBLocation.Text, '\');
  ShellExecute(0, 'open', PChar(Path), nil, nil, SW_SHOW);
end;

procedure TfrmGameReview.memoCommentChange(Sender: TObject);
begin
  game^.comment := memoComment.Text;
end;

procedure TfrmGameReview.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  icd.SaveCoords(self);
end;

procedure TfrmGameReview.vtScoresPaintText(Sender: TBaseVirtualTree;
  const TargetCanvas: TCanvas; Node: PVirtualNode; Column: TColumnIndex;
  TextType: TVSTTextType);
var
  p: PEntry;
begin
  p := Sender.GetNodeData(Node);
  if (column = 1) and (p^.nick = game^.host) then
    TargetCanvas.Font.Color := clBlue;
end;

procedure TfrmGameReview.btnSubmitClick(Sender: TObject);
begin
  MessageDlg('Submitting scores will be implemented in WC3Banlist 3.1.'
    +#13+#10+'Until then you only can give ratings for your own usage.'
    +#13+#10+''+#13+#10+'Also a /printscores command will be available then, which will show a table'+#13+#10+'of scores inside of the game to the other players, while you can see the scores'+#13+#10+'direcly in the lobby at the area where the map image is displayed normally.', mtWarning, [mbOK], 0);
end;

procedure TfrmGameReview.vtScoresClick(Sender: TObject);
begin
  SetButtonsEnabled;
end;

end.
