unit Unit3;

interface

uses
  Inifile, Classes, idhttp, Sysutils, Forms, IdMultipartFormData, ProxyInfo,
  VirtualTrees, WC3Types, IdComponent, Windows, IdCompressorZLibEX, ZlibEx,
  UListContainer;

const
  tasks: array[0..2] of string = (
    'logging in to buddies'' banlists',
    'uploading buddies'' banlists',
    'downloading buddies'' banlists');

type
  TBanlistThread = class(TThread)
  private
    t: tstringlist;
    err: string;
    sessionid: string;
    ver: string;
    task: Integer;
    page: Integer;
    kb: Integer;
    // objects
    IdHTTP1: TIdHTTP;
    mfd: TStringStream;
    IdCompressorZlibEx1: TIdCompressorZLibEx;
  protected
    procedure Execute; override;
    procedure DownloadBanlist;
    procedure ShowError;
    procedure PasteEntriesOfBuddyList;
    procedure GetLocalDatabase;
    procedure IdHTTP1Work(Sender: TObject; AWorkMode: TWorkMode;
      AWorkCount: Integer);
    procedure SetProgress;
    procedure Init;
    procedure Finish;
  public
    // Parameters
    doBuddyBanlist: boolean;
    parent: THandle;
    URLLogin: string;
    URLDownload: string;
    URLUpload: string;
    login: string;
    pass: string;
    // Methods
    procedure Cancel;
  end;

implementation

uses unit1, Controls;

procedure TBanlistThread.IdHTTP1Work(Sender: TObject;
  AWorkMode: TWorkMode; AWorkCount: Integer);
begin
  if Terminated then IdHTTP1.Disconnect else
  begin
    kb := AWorkCount shr 10;
    Synchronize(SetProgress);
  end;
end;

procedure TBanlistThread.DownloadBanlist;
var
  a: string;
  i: integer;
  mp: TIdMultiPartFormDataStream;
  StringStream: TStringStream;
begin
  if doBuddyBanlist then
  begin
    t.Clear;
    page := 1;
    task := 0;
    // log in
    try
      sessionid := IdHTTP1.Get(
        URLLogin +
        '?login=' + login +
        '&pass=' + pass +
        '&clientversion=' + HTTPEncode(ver));
      if Terminated then exit;
    except
      on e: EIdHTTPProtocolException do
      begin
        err := e.Message + cr + StringReplace(e.ErrorMessage, #$A, #$0D#$0A,
          [rfReplaceAll]);
        Synchronize(ShowError);
        Exit;
      end;
      on e: Exception do
      begin
        err := e.Message;
        Synchronize(ShowError);
        Exit;
      end;
    end;
    // check for success
    if pos(' ', sessionid) <> 0 then
    begin
      err := sessionid;
      Synchronize(ShowError);
      exit;
    end;
    // upload mine
    task := 1;
    Synchronize(GetLocalDatabase);
    if t.count > 0 then
    begin
      a := '';
      for i := 0 to t.Count - 2 do
        a := a + t[i] + #13#10;
      a := a + t[t.count - 1];
{      mfd.WriteString('bans=' + HTTPEncode(a));
      mfd.WriteString('&sid=' + sessionid);
      mfd.WriteString('&clientversion=' + ver);
      mfd.Free;
      mfd := TStringStream.Create('');}
      StringStream := TStringStream.Create(a);
      try
        ZCompressStream(StringStream, mfd);
      finally
        StringStream.Free;
      end;

      mp := TIdMultiPartFormDataStream.Create;
      mp.AddFormField('sid', sessionid);
      mp.AddFormField('clientversion', ver);
      mp.AddFormField('bans', mfd.DataString);
      mp.AddFormField('compressed', 'yes');
      mp.Position := 0;
      try
        IdHTTP1.Request.ContentType := mp.RequestContentType;
        a := IdHTTP1.Post(URLUpload, mp);
        mp.Free;
        if Terminated then exit;
      except
        on e: EIdHTTPProtocolException do
        begin
          err := e.Message + cr + StringReplace(e.ErrorMessage, #$A, #$0D#$0A,
            [rfReplaceAll]);
          Synchronize(ShowError);
          mp.Free;
        end;
        on e: Exception do
        begin
          err := e.Message;
          Synchronize(ShowError);
          mp.Free;
        end;
      end;
    end;
    // download buddies' lists
    task := 2;
    // set progress to zero
    kb := 0;
    Synchronize(SetProgress);
    try
      t.Text := IdHTTP1.Get(URLDownload + '?list=others&sid=' + sessionid);
      if Terminated then exit;
      Synchronize(PasteEntriesOfBuddyList);
    except
      on e: EIdHTTPProtocolException do
      begin
        err := e.Message + cr + StringReplace(e.ErrorMessage, #$A, #$0D#$0A,
          [rfReplaceAll]);
        Synchronize(ShowError);
        Exit;
      end;
      on e: Exception do
      begin
        err := e.Message;
        Synchronize(ShowError);
        exit;
      end;
    end;
  end;
end;

procedure TBanlistThread.Execute;
begin
  Init;
  try
    exchangingBanlist := true;
    DownloadBanlist;
    exchangingBanlist := false;
  except
    on e: Exception do
    begin
      err := e.Message;
      Synchronize(ShowError);
      exchangingBanlist := false;
    end;
  end;
  Finish;
end;

procedure TBanlistThread.GetLocalDatabase;
var
  i: integer;
  a: string;
  ban: PBan;
  info: PInfo;
begin
  if terminated then
    Exit;
  t.Clear;
  // bans
  for i := 0 to ListContainer.banlist.list.count - 1 do
  begin
    ban := ListContainer.banlist.list[i];
    if not CheckNicknameSpelling(ban^.nick) then
    begin
      t.clear;
      IniInfo('Error while uploading your local banlist to banlist.nl ...' + cr +
        'Illegal character in nickname: ' + ban^.nick + cr +
        'Delete this ban and empty the recycle bin to have it permanently removed.');
      break;
    end else
    if length(ban^.nick) > 15 then
    begin
      t.clear;
      IniInfo('Error while uploading your local banlist to banlist.nl ...' + cr
        +
        'Nickname length exceeds the maximum of 15 characters: ' + ban^.nick + cr +
        'Delete this ban and empty the recycle bin to have it permanently removed.');
      break;
    end
    else
    begin
      a := ban^.nick + #9 + Copy(ban^.comment, 1, 255) + #9 + IntToStr(Ord(ban^.deleted));
      t.Add(a);
    end;
  end;
  // infos
  for i := 0 to ListContainer.infolist.list.count - 1 do
  begin
    info := ListContainer.infolist.list[i];
    if not CheckNicknameSpelling(info^.nick) then
    begin
      t.clear;
      IniInfo('Error while uploading your local infolist to banlist.nl ...' + cr +
        'Illegal character in nickname: ' + info^.nick + cr +
        'Delete this info and empty the recycle bin to have it permanently removed.');
      break;
    end else
    if length(info^.nick) > 15 then
    begin
      t.clear;
      IniInfo('Error while uploading your local infolist to banlist.nl ...' + cr
        +
        'Nickname length exceeds the maximum of 15 characters: ' + info^.nick + cr +
        'Delete this info and empty the recycle bin to have it permanently removed.');
      break;
    end else
    begin
      a := info^.nick + #9 + Copy(info^.comment, 1, 255) + #9 + IntToStr(2 + Ord(info^.deleted));
      t.Add(a);
    end;
  end;
end;

procedure TBanlistThread.PasteEntriesOfBuddyList;
var
  s: tstringlist;
  i: integer;
  p: PBanBuddiesBanlist;
begin
  if terminated then
    Exit;
  s := tstringlist.Create;
  with WC3Banlist_Main do
  try
    vstBuddy.Clear;
    try
      vstBuddy.BeginUpdate;
      for i := 0 to t.Count - 1 do
      begin
        Split(t[i], #9, s);
        try
          p := buddy_addEntry;
          p^.nickname := s[0];
          p^.comment := s[1];
          p^.source := s[2];
          // p^.ImageIndex := StrToInt(s[3]);
          p^.imageindex := -1; // no icon because tab already has one
        except
          raise Exception.Create('Format failure in line ' + inttostr(i) + #13#10
            + '[' + t[i] + ']');
        end;
      end;
    finally
      vstBuddy.EndUpdate;
    end;
    pgblbuddylist.caption := 'Buddies'' Banlists (' +
      inttostr(vstBuddy.TotalCount) + ')';
  finally
    s.free;
  end;
end;

procedure TBanlistThread.ShowError;
begin
  if Terminated then Exit;
  with WC3Banlist_Main do
  begin
    memoError.Lines.add('Error while ' + tasks[task] + ' ...');
    memoError.Lines.add(err);
    pages.ActivePage := tabOnlineBanlists;
    pgblbuddylist.caption := 'Buddies'' Banlists (error)';
    panError.Visible := True;
    Application.Processmessages;
  end;
end;

procedure TBanlistThread.SetProgress;
begin
  with WC3Banlist_Main do
    case task of
        1: pgBlBuddyList.Caption := 'Buddies'' Banlists (UL: ' + inttostr(kb) +
          ' kB)';
        2: pgBlBuddyList.Caption := 'Buddies'' Banlists (DL: ' + inttostr(kb) +
          ' kB)';
    end;
end;

procedure TBanlistThread.Cancel;
begin
  Terminate;
  // very brual
  if exchangingBanlist and (IdHTTP1 <> nil) and (IdHTTP1.Connected) then
    IdHTTP1.Disconnect;
end;

procedure TBanlistThread.Init;
var
  proxy: String;
  port: Integer;
begin
  IdHTTP1 := tidhttp.create(nil);
  IdCompressorZlibEx1 := TIdCompressorZLibEx.Create;
  IdHTTP1.Compressor := IdCompressorZlibEx1;
  IdHTTP1.Request.AcceptEncoding := 'gzip, deflate, ';
//  IdHTTP1.Request.ContentEncoding := 'deflate';
  IdHTTP1.ReadTimeout := 30000;
  IdHTTP1.ConnectTimeout := 10000;
  IdHTTP1.OnWork := IdHTTP1Work;
  if WC3Banlist_Main.chkproxy.Checked then
  begin
    getproxyserver('http', proxy, port);
    IdHTTP1.ProxyParams.ProxyServer := proxy;
    IdHTTP1.ProxyParams.ProxyPort := port;
  end
  else
  begin
    IdHTTP1.ProxyParams.ProxyServer := '';
    IdHTTP1.ProxyParams.ProxyPort := 0;
  end;
  t := TStringList.Create;
  t.Delimiter := ' ';
  ver := CURRENT_VERSION;
  // ver := HTTPEncode(ver);
  mfd := TStringStream.Create('');
end;

procedure TBanlistThread.Finish;
begin
  mfd.Free;
  t.Free;
  IdCompressorZlibEx1.Free;
  IdHTTP1.Free;
end;

end.

