unit Unit4;

interface

uses
  Classes, Windows, Clipbrd, SyncObjs;

type
  ClipThread = class(TThread)
  private
    { Private-Deklarationen }
  protected
    procedure Execute; override;
  public
    hSem: THandle;
    csText: TCriticalSection;
    text: string;
    TimeThreshold: Cardinal;
    destructor Destroy; override;
    procedure Init;
    procedure SetText(t: string);
    function GetText: String;
    procedure Clear;
  end;

implementation

{ ClipThread }

procedure ClipThread.Clear;
begin
  csText.Enter;
  text := '';
  csText.Leave;
end;

destructor ClipThread.Destroy;
begin
  CloseHandle(hSem);
  hSem := 0;
  csText.Free;
  inherited;
end;

procedure ClipThread.Execute;
begin
  while not terminated do
  begin
    if WaitForSingleObject(hSem, 500) <> WAIT_TIMEOUT then
    begin
      if TimeThreshold > 0 then
        Sleep(TimeThreshold);

      csText.Enter;
      try
        if text <> '' then
        begin
          Clipboard.AsText := text;
          text := '';
        end;
      finally
        csText.Leave;
      end;

      WaitForsingleObject(hSem, 0);
    end;
  end;
end;

function ClipThread.GetText: String;
begin
  csText.Enter;
  try
    if text <> '' then
      result := text
    else
      result := Clipboard.AsText;
  finally
    csText.Leave;
  end;
end;

procedure ClipThread.Init;
begin
  hSem := CreateSemaphore(nil, 0, 1, nil);
  csText := TCriticalSection.Create;
  text := '';
  TimeThreshold := 0;
end;

procedure ClipThread.SetText(t: string);
begin
  csText.Enter;
  if text <> '' then
    text := text + ' ';
  text := text + t;
  ReleaseSemaphore(hSem, 1, nil);
  csText.Leave;
end;

end.
