unit Wizard;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ExtCtrls, ComCtrls, CommCtrl, StdCtrls, ShellApi, ImgList,
  Registry;

const
  TitleCount = 5;
  Titles: array[0 .. TitleCount - 1] of string =
  (
    '',
    'NIC Selection',
    'Banlist.nl Account',
    'Games Selection',
    'Warcraft III Installation Directory'
  );

  SubTitles: array[0 .. TitleCount - 1] of string =
  (
    '',
    'Choose the network adapter which connects you to the internet.',
    'Enable sharing of your list through the Banlist.nl Exchange System.',
    'Customize WC3Banlist for the games you play.',
    'Allow WC3Banlist to obtain and organize replays.'
  );

type
  TWizardInput = class
  public
    NICs: array of string;
    AutoDetectNicMethod: procedure of object;
    BrowseForWar3DirMethod: function(var dir: string): boolean of object;
  end;

  TWizardOutput = class
  public
    NICIndex: integer;
    UseBanlistnl: boolean;
    BanlistnlUser: string;
    BanlistnlPass: string;
    PlaysAzerothPubs: boolean;
    PlaysLordaeronPubs: boolean;
    PlaysKalimdorPubs: boolean;
    PlaysNorthrendPubs: boolean;
    PlaysAzerothTDA: boolean;
    PlaysLordaeronTDA: boolean;
    PlaysNorthrendDCE: boolean;
    WarcraftDir: string;
  end;
  
  TWizardForm = class(TForm)
    ButtonPanel: TPanel;
    Pages: TPageControl;
    WelcomePage: TTabSheet;
    WelcomeImage: TImage;
    WelcomePanel: TPanel;
    CancelButton: TButton;
    NextButton: TButton;
    BackButton: TButton;
    NICSelectionPage: TTabSheet;
    WebLabel: TLabel;
    AutoDetectNICButton: TButton;
    NICCombo: TComboBoxEx;
    SmallImages: TImageList;
    WelcomeTitleLabel: TLabel;
    WelcomeLabel: TLabel;
    BanlistnlPage: TTabSheet;
    BanlistnlYesRadio: TRadioButton;
    NICLabel1: TLabel;
    NICLabel2: TLabel;
    BanlistnlLabel1: TLabel;
    BanlistnlCreateAccButton: TButton;
    BanlistnlLabel2: TLabel;
    BanlistnlUserEdit: TEdit;
    BanlistnlPwdEdit: TEdit;
    BanlistnlUserEditLabel: TLabel;
    BanlistnlPwdEditLabel: TLabel;
    BanlistnlNoRadio: TRadioButton;
    RealmPage: TTabSheet;
    RealmAzCheck: TCheckBox;
    RealmLabel: TLabel;
    RealmAzTDACheck: TCheckBox;
    RealmNoCheck: TCheckBox;
    RealmKaCheck: TCheckBox;
    RealmNoDCECheck: TCheckBox;
    RealmLoCheck: TCheckBox;
    RealmPubLabel: TLabel;
    RealmPrivLabel: TLabel;
    RealmLoTDACheck: TCheckBox;
    WarcraftPage: TTabSheet;
    TitlePanel: TPanel;
    TitleImage: TImage;
    TitleLabel: TLabel;
    SubTitleLabel: TLabel;
    WarcraftLabel: TLabel;
    WarcraftEdit: TEdit;
    WarcraftBrowseButton: TButton;
    ButtonBevel: TBevel;
    TitleBevel: TBevel;
    procedure WarcraftEditChange(Sender: TObject);
    procedure WarcraftBrowseButtonClick(Sender: TObject);
    procedure RealmCheckClick(Sender: TObject);
    procedure BanlistnlPwdEditExit(Sender: TObject);
    procedure BanlistnlEditChange(Sender: TObject);
    procedure BanlistnlUserEditKeyPress(Sender: TObject; var Key: Char);
    procedure BanlistnlUserEditExit(Sender: TObject);
    procedure BanlistnlRadioClick(Sender: TObject);
    procedure BanlistnlCreateAccButtonClick(Sender: TObject);
    procedure PagesChange(Sender: TObject);
    procedure NICComboChange(Sender: TObject);
    procedure AutoDetectNICButtonClick(Sender: TObject);
    procedure WebLabelClick(Sender: TObject);
    procedure NextButtonClick(Sender: TObject);
    procedure BackButtonClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    Input: TWizardInput;
    Output: TWizardOutput;
    OldPagesWndProc: TWndMethod;
    Reg: TRegistry;
    procedure NewPagesWndProc(var Message:TMessage);
  public
    constructor Create(AOwner: TComponent; AInput: TWizardInput;
      AOutput: TWizardOutput); reintroduce;
  end;
  
  TWizard = class
  private
    WizardForm: TWizardForm;
  public
    Input: TWizardInput;
    Output: TWizardOutput;
    function Execute: Integer;
    constructor Create;
    destructor Destroy; override;
  end;

implementation

{$R *.dfm}

{ TWizard }

constructor TWizard.Create;
begin
  inherited;
  Input := TWizardInput.Create;
  Output := TWizardOutput.Create;
end;

destructor TWizard.Destroy;
begin
  Output.Free;
  Input.Free;
  inherited;
end;

function TWizard.Execute: Integer;
begin
  WizardForm := TWizardForm.Create(Application, Input, Output);
  Result := WizardForm.ShowModal;
  WizardForm.Free;
end;

{ TWizardForm }

constructor TWizardForm.Create(AOwner: TComponent; AInput: TWizardInput;
  AOutput: TWizardOutput);
begin
  Input := AInput;
  Output := AOutput;
  inherited Create(AOwner);
end;

procedure TWizardForm.AutoDetectNICButtonClick(Sender: TObject);
begin
  if Assigned(Input.AutoDetectNicMethod) then
    Input.AutoDetectNicMethod;
end;

procedure TWizardForm.BackButtonClick(Sender: TObject);
begin
  Pages.SelectNextPage(False, False);
end;

procedure TWizardForm.BanlistnlRadioClick(Sender: TObject);
begin
  BanlistnlUserEdit.Enabled := BanlistnlYesRadio.Checked;
  BanlistnlPwdEdit.Enabled := BanlistnlYesRadio.Checked;
  BanlistnlUserEditLabel.Enabled := BanlistnlYesRadio.Checked;
  BanlistnlPwdEditLabel.Enabled := BanlistnlYesRadio.Checked;
  Output.UseBanlistnl := BanlistnlYesRadio.Checked;
  BanlistnlEditChange(Sender);
end;

procedure TWizardForm.BanlistnlEditChange(Sender: TObject);
begin
  NextButton.Enabled := BanlistnlNoRadio.Checked or
    ((BanlistnlUserEdit.Text <> '') and (BanlistnlPwdEdit.Text <> ''));
end;

procedure TWizardForm.BanlistnlUserEditExit(Sender: TObject);
begin
  Output.BanlistnlUser := (Sender as TEdit).Text;
end;

procedure TWizardForm.BanlistnlUserEditKeyPress(Sender: TObject; var Key: Char);
begin
  if not (UpCase(Key) in ['A'..'Z','0'..'9',Chr(VK_BACK),Chr(VK_DELETE)]) then
    Key := #0;
end;

procedure TWizardForm.BanlistnlCreateAccButtonClick(Sender: TObject);
begin
  ShellExecute(Application.Handle, 'open', 'http://www.banlist.nl/signup.php',
    '', '', SW_SHOW);
end;

procedure TWizardForm.BanlistnlPwdEditExit(Sender: TObject);
begin
  Output.BanlistnlPass := (Sender as TEdit).Text;
end;

procedure TWizardForm.FormCreate(Sender: TObject);
var
  i: Integer;
  citem: TComboExItem;
  s: string;
begin
  OldPagesWndProc := Pages.WindowProc;
  Pages.WindowProc := NewPagesWndProc;
  Height := Height - 10;
  Width := Width - 8;
  Pages.ActivePage := WelcomePage;
  for i := 0 to Length(Input.NICs) - 1 do
  begin
    citem := NICCombo.ItemsEx.Add;
    citem.Caption := Input.NICs[i];
    citem.ImageIndex := 0;
  end;
  NICCombo.ItemIndex := 0;
  Reg := TRegistry.Create(KEY_QUERY_VALUE);
  try
    Reg.RootKey := HKEY_LOCAL_MACHINE;
    if Reg.OpenKey('Software\Blizzard Entertainment\Warcraft III', false) then
    begin
      s := Reg.ReadString('InstallPath');
      if (s <> '') and FileExists(s + '\war3.mpq') then
        WarcraftEdit.Text := s;
    end
    else
    begin
      Reg.RootKey := HKEY_CURRENT_USER;
      if Reg.OpenKey('Software\Blizzard Entertainment\Warcraft III', false) then
      begin
        s := Reg.ReadString('InstallPath');
        if (s <> '') and FileExists(s + '\war3.mpq') then
          WarcraftEdit.Text := s;
      end
      else
        WarcraftEdit.Text := '';
    end;
  finally
    Reg.Free;
  end;
end;

procedure TWizardForm.WarcraftBrowseButtonClick(Sender: TObject);
var
  s: string;
begin
  if Assigned(Input.BrowseForWar3DirMethod) and
    Input.BrowseForWar3DirMethod(s) then
  begin
    if FileExists(s + '\war3.mpq') then
      WarcraftEdit.Text := s
    else
      Application.MessageBox('The directory selected is not the Warcraft III ' +
        'installation directory!', 'Error', 16);
  end;
end;

procedure TWizardForm.WarcraftEditChange(Sender: TObject);
begin
  if FileExists(WarcraftEdit.Text + '\war3.mpq') then
  begin
    NextButton.Enabled := true;
    Output.WarcraftDir := WarcraftEdit.Text;
  end
  else
  begin
    NextButton.Enabled := false;
    Output.WarcraftDir := '';
  end;
end;

procedure TWizardForm.WebLabelClick(Sender: TObject);
begin
  ShellExecute(Application.Handle, 'open', 'http://www.wc3banlist.de', '', '',
    SW_SHOW);
end;

procedure TWizardForm.NewPagesWndProc(var Message: TMessage);
begin
  if Message.Msg = TCM_ADJUSTRECT then
  begin
    OldPagesWndProc(Message);
    PRect(Message.LParam)^.Left   := 0;
    PRect(Message.LParam)^.Right  := ClientWidth;
    PRect(Message.LParam)^.Top    := PRect(Message.LParam)^.Top - 6;
    PRect(Message.LParam)^.Bottom := ClientHeight;
  end
  else
    OldPagesWndProc(Message);
end;

procedure TWizardForm.NextButtonClick(Sender: TObject);
begin
  Pages.SelectNextPage(True, False);
end;

procedure TWizardForm.NICComboChange(Sender: TObject);
begin
  Output.NICIndex := NICCombo.ItemIndex;
end;

procedure TWizardForm.PagesChange(Sender: TObject);
begin
  NextButton.Enabled := True;
  NextButton.Caption := '&Next >';
  NextButton.ModalResult := mrNone;
  BackButton.Enabled := True;
  TitlePanel.Parent := Pages.ActivePage;
  TitlePanel.Visible := True;
  case Pages.ActivePageIndex of
    0:
    begin
      BackButton.Enabled := False;
      TitlePanel.Visible := False;
    end;
    2: BanlistnlRadioClick(Sender);
    4: WarcraftEditChange(Sender);
  end;
  TitleLabel.Caption := Titles[Pages.ActivePageIndex];
  SubTitleLabel.Caption := SubTitles[Pages.ActivePageIndex];
  if Pages.ActivePageIndex = Pages.PageCount - 1 then
  begin
    NextButton.Caption := '&Finish';
    NextButton.ModalResult := mrOK;
  end;
end;

procedure TWizardForm.RealmCheckClick(Sender: TObject);
begin
  case (Sender as TCheckBox).Tag of
    0: Output.PlaysAzerothPubs := (Sender as TCheckBox).Checked;
    1: Output.PlaysLordaeronPubs := (Sender as TCheckBox).Checked;
    2: Output.PlaysNorthrendPubs := (Sender as TCheckBox).Checked;
    3: Output.PlaysKalimdorPubs := (Sender as TCheckBox).Checked;
    4: Output.PlaysAzerothTDA := (Sender as TCheckBox).Checked;
    5: Output.PlaysLordaeronTDA := (Sender as TCheckBox).Checked;
    6: Output.PlaysNorthrendDCE := (Sender as TCheckBox).Checked;
  end;
end;

end.
