{
! This is not the original TZniffer, it's a modified version by Ghost_3k !
! ghost_3k@gmx.net - http://www.misalpina.com/ghost3k !
********************************************************************************
--------------------------------------------------------------------------------
                                    TZNIFFER

                for Packet Capture Driver by Politecnico di Torino

                         Written by Lars Peter Christiansen
--------------------------------------------------------------------------------
 TERMS AND CONDITIONS OF USE.
 All of this software is Copyright(C) 2002 Lars Peter Christiansen.

 The author of this software assumes no liability for damages caused under
 any circumstances whatsoever, and is under no obligation. Use of the software
 indicates acceptance of all conditions contained in this document. If you do
 not agree to these terms, you must delete this software immediately.

 You may distribute the archive in which this software is distributed, but
 under no circumstances must this archive be changed. Distributing a modified
 archive is a violation of the software license.

 If you do redistribute this software, please let me know at the email address
 given below.

 If you have any questions, requests, bug reports, etc., please contact me at
 the address given below.

Lars Peter Christiansen
Email  : bakkevej@stofanet.dk
Website: http://home1.stofanet.dk/nitezhifter

--------------------------------------------------------------------------------

                          [ user application ]
                          [     TZniffer     ] <- you are here!
                          [       PCAP       ]
                          [    Netadapter    ]

*** Changes by Ghost_3k - ghost_3k@gmx.net ***

   March 2002 - added Promisc property
February 2002 - added driver_installed (check it to see if winpcap dlls were succesfully loaded)
 January 2002 - added SetFilter function

*** Changes by Danny Grobe - gdanny@gmx.de ***

     May 2004 - added support for WinPCap 3.1beta

********************************************************************************
}
unit Zniffer;

interface
uses Windows,
  Classes,
  Sysutils,
  Pcap, bpf,
  Packet32,
  RegExpr,
  WinSock;
type

  PETHERNET_HDR = ^ETHERNET_HDR;
  ETHERNET_HDR = packed record
    Destination: array[0..5] of UCHAR;
    Source: array[0..5] of UCHAR;
    Protocol: array[0..1] of UCHAR;
    Data: array[0..0] of UCHAR;
  end;

  PIP_RHDR = ^IP_RHDR;
  IP_RHDR = packed record
    Verlen: UCHAR; //4bit version 4bit length (bytes/8)
    Service: UCHAR; // TOS
    Length: word;
    Ident: word;
    Flagoff: array[0..1] of UCHAR; //3bit flag - 13 bit offset
    TimeLive: UCHAR;
    Protocol: UCHAR;
    Checksum: word;
    SrcIP: Cardinal;
    DestIP: Cardinal;
    Data: array[0..0] of UCHAR;
  end;

  // Added ( Lars Peter Christiansen 13-04-2001)
  PTCP_RHDR = ^TCP_RHDR;
  TCP_RHDR = packed record
    SrcPort: WORD;
    DestPort: WORD;
    SequenceNr: array[0..3] of UCHAR;
    AckNumber: array[0..3] of UCHAR;
    LenResvFlags: array[0..1] of UCHAR; //length(4bits) rsvd(6bits) flags(6bits)
    WindowSize: array[0..1] of UCHAR;
    Checksum: array[0..1] of UCHAR;
    UrgentPtr: array[0..1] of UCHAR;
    Data: array[0..0] of UCHAR;
  end;

  PUDP_RHDR = ^TUDP_RHDR;
  TUDP_RHDR = packed record
    SrcPort: Word;
    DestPort: Word;
    Size: Word;
    Checksum: Word;
    Data: array[0..0] of UCHAR;
  end;

  Tzniffer = class;

  // Thread that listens to selected Netadapter
  TZnifferThread = class(Tthread)
  private
    Z: Tzniffer;
  public
    ReadTimes: integer;
    constructor Create(Zniffer: Tzniffer);
    procedure Execute; override;
  end;

  // The main TZniffer Class here
  TZniffer = class(TObject)
  private
    Fadapters: TstringList; // the adapters found on the system
    FadapterIndex: Integer; // current adapter
    Fsnooping: Boolean; // Flag indicating snooping activity
    Fpromisc: boolean; // Flag indicating if promiscous or directed mode
    FAdapterDesc: TStringlist;
    FAddresses: TStringList;
    FAddress: Cardinal;
    FSnapLen: Integer;
    FPackets: Cardinal;
    FFilter: String;
    FThread: TZnifferThread; // The listening thread
    procedure ThreadTerminate(Sender: tobject);
    procedure SetAdapterIndex(const Value: integer);
    procedure SetPromisc(const Value: boolean);
    procedure SetSnapLen(const Value: integer);
  public
    FAdapter: Padapter; // Handle to the pcapdriver
    driver_installed: boolean;
    OnPacket: procedure(Data: pointer; recvbytes: Word; utickcount: Int64) of object;
    constructor Create;
    destructor Destroy; override;
    function SetFilter(filter: string): byte;
    function Activate(var ErrStr: string): boolean;
    function Deactivate(var ErrStr: string): boolean;
    procedure ReadAdapterList;
    property Snooping: boolean read Fsnooping;
    property Adapters: TstringList read Fadapters;
    property AdapterDesc: TStringlist read FAdapterDesc;
    property Addresses: TStringList read FAddresses;
    property AdapterIndex: integer read FadapterIndex write SetAdapterIndex;
    property Promisc: boolean write SetPromisc;
    property SnapLen: Integer read FSnapLen write SetSnapLen;
    property Thread: TZnifferThread read FThread;
    property Filter: String read FFilter;
    property Packets: Cardinal read FPackets write FPackets;
  end;

const //Look in rfc1340 dokument
  PROTO_IP = $0800;

function TOUSHORT(x: PChar): SHORT; // Ripped C Makro
implementation

function TOUSHORT(x: PChar): SHORT;
begin
  Result := (SHORT(x^) shl 8) or (SHORT((x + 1)^)); // FP May 10, 1999
end;

{ TZniffer }

constructor TZniffer.Create;
begin
  FAdapters := Tstringlist.Create;
  FAdapterDesc := Tstringlist.Create;
  FAddresses := TStringList.Create;
  FadapterIndex := 0;
  FPackets := 0;
  FSnapLen := DEFAULT_SNAPLEN;
  FAdapter := nil;
  Fsnooping := False;
  Fpromisc := true;
  if pcap.ok and packet32.ok then
  begin
    driver_installed := true;
    ReadAdapterList;
  end
  else
    driver_installed := false;
end;

destructor TZniffer.Destroy;
var
  E: string;
begin
  DeActivate(E);
  FreeAndNil(FAdapters);
  FreeAndNil(FAdapterDesc);
  FreeAndNil(FAddresses);
  inherited;
end;

//------------------------------------------------------------------------------
//  ACTIVATE SNOOP AND START READTHREAD
//------------------------------------------------------------------------------

function TZniffer.Activate(var ErrStr: string): boolean;
var
  err: array[0..255] of char;
  ipaddress: String;
  flags: integer;
begin
  Result := false;

  fillchar(err, sizeof(err), 0);
  // Check if Snooping is active
  if Fsnooping or (Fadapter <> nil) then
  begin
    ErrStr := 'Snooping already activated';
    exit;
  end;

  flags := Integer(FPromisc);
  // Open Driver and NetAdapter
  FAdapter := Pcap.pcap_open(Pchar(FAdapters[FAdapterindex]), FSnapLen, flags,
    100, nil, err);
  if FAdapter = nil then
  begin
    ErrStr := err;
    exit;
  end;

  if not Assigned(OnPacket) then
  begin
    ErrStr := 'No Packet Read Callback function assigned';
    exit;
  end;

  ipaddress := FAddresses[FAdapterindex];
  FAddress := 0;
  FAddress := inet_addr(PChar(ipaddress));

  // Start Snoop Read Thread
  FThread := TZnifferThread.create(self);
  Fthread.ReadTimes := 10;
  Fthread.OnTerminate := ThreadTerminate;
  Fthread.FreeOnTerminate := false;
  Fthread.resume;
  FSnooping := True;
  if FFilter <> '' then
    SetFilter(FFilter);
  Result := true;
end;

function TZniffer.Deactivate(var ErrStr: string): boolean;
begin
  result := false;

  if (not Fsnooping) then
  begin
    errstr := 'Snooping not active';
    exit;
  end;
  if FThread = nil then
  begin
    errstr := 'No thread to stop';
    exit;
  end;

  // Stop Snooping Thread
  FThread.Terminate;
  FThread.WaitFor;
  FThread.Free;
  Fthread := nil;

  // Release Driver Handle
//  _Pcap_Close(FAdapter); // buggy
  FAdapter := nil;

  result := true;
end;

//------------------------------------------------------------------------------
//  { TZnifferThread }
//
//  And Snoop Driver Callback function which cannot be procedure of object!
//------------------------------------------------------------------------------

procedure CaptureCB(User: pointer; const Header: Ppcap_pkthdr; const Data:
  pchar);
begin
  TZniffer(user).OnPacket(Data, header^.len, header^.ts.tv_Sec * 1000000 + header^.ts.tv_uSec);
end;

constructor TZnifferThread.Create(Zniffer: Tzniffer);
begin
  ReadTimes := 0;
  Z := Zniffer;
  inherited Create(TRUE);
end;

procedure TZnifferThread.Execute;
var
  hdr: ppcap_pkthdr;
  p: pointer;
begin
  if Z = nil then
    exit;

  while not Terminated do
    if Pcap_Next_ex(Z.FAdapter, hdr, p) = 1 then
    begin
      Z.OnPacket(p, hdr^.len, hdr^.ts.tv_Sec * 1000000 + hdr^.ts.tv_uSec);
      inc(Z.FPackets);
    end;
end;

procedure TZniffer.ThreadTerminate(Sender: tobject);
begin
  Fsnooping := false;
end;

procedure TZniffer.SetAdapterIndex(const Value: integer);
begin
  if (value > -1) and (value < Adapters.count) then
    FadapterIndex := Value;
end;

procedure TZniffer.SetPromisc(const Value: boolean);
begin
  Fpromisc := value;
end;

function TZniffer.SetFilter(filter: string): byte;
var
  fp: pbpf_program;
begin
  result := 0;
  New(fp);
  if pcap_compile_nopcap(1000000, 1, fp, pchar(filter), 1, $FFFFFF) = 0 then
  begin
    result := 1;
    if pcap_setfilter(FAdapter, fp) = 0 then
      result := 2;
  end;
  if result = 2 then
    FFilter := filter;
  Dispose(fp);
end;

procedure TZniffer.ReadAdapterList;
var
  errbuff: array[0..PCAP_ERRBUF_SIZE] of char;
  name, descr: string;
  i: integer;
  suffix: string;
  re: TRegExpr;
  alldevs, dev: Ppcap_if_t;
  addr: Ppcap_addr;
  saddr: String;
begin
  FAdapterDesc.Clear;
  FAdapters.Clear;
  FAddresses.Clear;
  pcap_findalldevs_ex('rpcap://', nil, alldevs, errbuff);
  dev := alldevs;
  re := TRegExpr.Create;
  re.Expression := 'Network adapter \''(.*)\'' on.*';
  try
    while dev <> nil do
    begin
      name := dev^.name;
      descr := dev^.description;
      addr := dev^.addresses;
      saddr := '';
      while addr <> nil do
      begin
        if inet_ntoa(addr.addr^.sin_addr) <> '0.0.0.0' then
          saddr := inet_ntoa(addr.addr^.sin_addr);
        addr := addr^.next;
      end;
      FAddresses.Add(trim(saddr));
      FAdapters.Add(name);
      if re.Exec(descr) then descr := re.Match[1];
      suffix := '';
      i := 1;
      while FAdapterDesc.IndexOf(descr + suffix) <> -1 do
      begin
        inc(i);
        suffix := ' #'+IntToStr(i);
      end;
      FadapterDesc.Add(descr + suffix);
      dev := dev^.next;
    end;
  finally
    re.Free;
    pcap_freealldevs(alldevs);
  end;
end;

procedure TZniffer.SetSnapLen(const Value: integer);
begin
  if (Value >= 1) then
    FSnapLen := Value
  else
    raise Exception.Create('illegal snaplen');
end;

end.

