unit raw_ping;

interface
uses
  Windows, SysUtils, Classes;

type
  TSunB = packed record
    s_b1, s_b2, s_b3, s_b4: byte;
  end;

  TSunW = packed record
    s_w1, s_w2: word;
  end;

  PIPAddr = ^TIPAddr;
  TIPAddr = record
    case integer of
      0: (S_un_b: TSunB);
      1: (S_un_w: TSunW);
      2: (S_addr: longword);
  end;

  IPAddr = TIPAddr;

  TIcmp_Echo_Reply = record
    Address: TIPAddr;
    Status: Cardinal;
    RoundTripTime: Cardinal;
    DataSize: Word;
    Reserved: Word;
    Data: Pointer;
  end;
  PIcmp_Echo_Reply = ^TIcmp_Echo_Reply;

var
  timeout: Cardinal = 1000;

function IcmpCreateFile: THandle; stdcall; external 'icmp.dll';
function IcmpCloseHandle(icmpHandle: THandle): boolean; stdcall; external
  'icmp.dll'
function IcmpSendEcho(
  IcmpHandle: THandle;
  DestinationAddress: IPAddr;
  RequestData: Pointer;
  RequestSize: Smallint;
  RequestOptions: pointer;
  ReplyBuffer: Pointer;
  ReplySize: DWORD;
  Timeout: DWORD): DWORD; stdcall; external 'icmp.dll';

function IcmpSendEcho2(
  IcmpHandle: THandle;
  Event: THandle;
  ApcRoutine: Pointer;
  APcContext: Pointer;
  DestinationAddress: IPAddr;
  RequestData: Pointer; RequestSize: Smallint;
  RequestOptions: pointer;
  ReplyBuffer: Pointer;
  ReplySize: DWORD;
  Timeout: DWORD): DWORD; stdcall; external 'icmp.dll';

function Ping(InetAddress: Cardinal): Integer;

procedure TranslateStringToTInAddr(AIP: string; var AInAddr);

implementation

uses
  WinSock;

function Fetch(var AInput: string; const ADelim: string = ' '; const ADelete:
  Boolean = true)
  : string;
var
  iPos: Integer;
begin
  if ADelim = #0 then
  begin
    // AnsiPos does not work with #0
    iPos := Pos(ADelim, AInput);
  end
  else
  begin
    iPos := Pos(ADelim, AInput);
  end;
  if iPos = 0 then
  begin
    Result := AInput;
    if ADelete then
    begin
      AInput := '';
    end;
  end
  else
  begin
    result := Copy(AInput, 1, iPos - 1);
    if ADelete then
    begin
      Delete(AInput, 1, iPos + Length(ADelim) - 1);
    end;
  end;
end;

procedure TranslateStringToTInAddr(AIP: string; var AInAddr);
var
  phe: PHostEnt;
  pac: PChar;
  GInitData: TWSAData;
begin
  WSAStartup($101, GInitData);
  phe := GetHostByName(PChar(AIP));
  if Assigned(phe) then
  begin
    pac := phe^.h_addr_list^;
    if Assigned(pac) then
    begin
      with TIPAddr(AInAddr).S_un_b do
      begin
        s_b1 := Byte(pac[0]);
        s_b2 := Byte(pac[1]);
        s_b3 := Byte(pac[2]);
        s_b4 := Byte(pac[3]);
      end;
    end
    else
    begin
      raise Exception.Create('Error getting IP from HostName');
    end;
  end
  else
  begin
    raise Exception.Create('Error getting HostName');
  end;
  WSACleanup;
end;

function Ping(InetAddress: Cardinal): Integer;
var
  Handle: THandle;
  InAddr: IPAddr;
  rep: array[0..127] of byte;
  reply: PIcmp_Echo_Reply;
const
  text : String = 'WC3Banlist::PING' + #0;
begin
  result := -2;
  Handle := IcmpCreateFile;
  if Handle = INVALID_HANDLE_VALUE then
    Exit;
  inAddr.S_addr := InetAddress;
  if IcmpSendEcho(Handle, inAddr, @text[1], Length(text) +1, nil, @rep, sizeof(rep), timeout) <> 0 then
  begin
    reply := @rep;
    result := reply^.RoundTripTime;
  end
  else
    result := -1;
  IcmpCloseHandle(Handle);
end;

end.
